/* $Id: dst.c,v 1.2 2001/06/29 16:04:29 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	USGS Digital streams (DST)...

-----------------------------------------------------------------------*/

#include "dst.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/

/*---------------------------------------------------------------------*/
VOID DSTSetDefaults(DST_ARGS * args)
{
	INT32 i;

	ASSERT(args != NULL);

	args->handle = ASYNC_INVALID;
	AsyncDefaultParams(&args->params);
	args->params.flow_ctrl = ASYNC_FLOW_NONE;

	args->sync = DST_DFL_SYNC;
	args->aux = DST_DFL_AUX;
	args->rate = DST_DFL_RATE;
	args->byte_order = DST_DFL_BO;
	args->encoding = DST_DFL_EN;
	args->aux_type = DST_DFL_AUX_TYPE;

	for (i = 0; i < DST_MAX_CHANNELS; i++) {
		args->channel[i].active = FALSE;
		memset(&args->channel[i].name, 0, sizeof(GRF_NAME));
		args->channel[i].counts_per_volt = DST_DFL_CPV;
		args->channel[i].dc_offset = DST_DFL_DC;
	}

	return;
};

/*---------------------------------------------------------------------*/
BOOL DSTEncodeFrame(DST_FRAME * frame, UINT8 order, VOID *ptr, size_t *n)
{
	UINT8 *bytes;
	size_t i, k;

	ASSERT(frame != NULL);
	ASSERT(ptr != NULL);
	ASSERT(n != NULL);

	if (*n < DST_FRAME_SIZE) {
		*n = 0;
		return FALSE;
	}

	i = 0;
	bytes = (UINT8 *)ptr;

	bytes[i++] = frame->sync;

	for (k = 0; k < DST_MAX_CHANNELS; k++) {
		if (order == DST_BO_MSBLSB) {
			bytes[i++] = (UINT8)((frame->channel[k] & 0xFF00) >> 8);
			bytes[i++] = (UINT8)(frame->channel[k] & 0x00FF);
		}
		else {
			bytes[i++] = (UINT8)(frame->channel[k] & 0x00FF);
			bytes[i++] = (UINT8)((frame->channel[k] & 0xFF00) >> 8);
		}
	}

	bytes[i++] = frame->aux;

	*n = i;

	return TRUE;
}

/*---------------------------------------------------------------------*/
BOOL DSTDecodeFrame(DST_FRAME * frame, UINT8 order, VOID *ptr, size_t *n)
{
	UINT8 *bytes;
	size_t i, k;

	ASSERT(frame != NULL);
	ASSERT(ptr != NULL);
	ASSERT(n != NULL);

	if (*n < DST_FRAME_SIZE)
		return FALSE;

	i = 0;
	bytes = (UINT8 *)ptr;

	frame->sync = bytes[i++];

	for (k = 0; k < DST_MAX_CHANNELS; k++) {
		if (order == DST_BO_MSBLSB) {
			frame->channel[k] = (INT16)(((UINT16)bytes[i] << 8) | (UINT16)bytes[i + 1]);
			i += 2;
		}
		else {
			frame->channel[k] = (INT16)((UINT16)bytes[i] | ((UINT16)bytes[i + 1] << 8));
			i += 2;
		}
	}

	frame->aux = bytes[i++];

	*n = i;

	return TRUE;
}
