/* $Id: dst.h,v 1.2 2001/06/29 16:04:29 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	USGS Digital streams (DST)...

-----------------------------------------------------------------------*/
#if !defined _DST_H_INCLUDED_
#define _DST_H_INCLUDED_

#include <platform.h>
#include <async.h>
#include "sock.h"
#include <grf.h>

/* Constants ----------------------------------------------------------*/
#define DST_MAX_CHANNELS	3
#define DST_FRAME_SIZE		8

#define DST_BO_MSBLSB		0				/* Byte order */
#define DST_BO_LSBMSB		1

#define DST_EN_TWOS			0				/* Two's complement */
#define DST_EN_OFFSET		1				/* Offset binary */

#define DST_AUX_FRAME_END	0				/* Aux byte is frame end */
#define DST_AUX_TIMING		1				/* Aux byte is timing/parity */

/* Defaults... */
#define DST_DFL_SYNC		0x0d
#define DST_DFL_AUX			0x0a
#define DST_DFL_RATE		100.0
#define DST_DFL_BO			DST_BO_MSBLSB
#define DST_DFL_EN			DST_EN_TWOS
#define DST_DFL_AUX_TYPE	DST_AUX_FRAME_END

#define DST_DFL_CPV			13107			/* 2^16 / 5.0 (+/- 2.5) volts */
#define DST_DFL_DC			0

/* Types --------------------------------------------------------------*/

/* Frame structure */
typedef struct TAG_DST_FRAME {
	UINT8 sync;
	INT16 channel[DST_MAX_CHANNELS];
	UINT8 aux;
} DST_FRAME;

typedef struct TAG_DST_ARGS {
	H_ASYNC handle;							/* Async port handle */
	ASYNC_PARAMS params;					/* Async port parameters */
	REAL64 rate;							/* Sampling rate */
	UINT8 byte_order;						/* Sample byte order, DST_BO_??? */
	UINT8 encoding;							/* Sample encoding, DST_EN_??? */
	UINT8 aux_type;							/* Type of aux byte, DST_AUX_??? */
	UINT8 sync;								/* Sync byte value */
	UINT8 aux;								/* Aux byte value, only if aux type=FRAME_END */
	struct {
		BOOL active;						/* Channel active flag */
		GRF_NAME name;						/* GRF channel name */
		INT32 counts_per_volt;				/* Counts/volt */
		INT32 dc_offset;					/* DC offset */
	} channel[DST_MAX_CHANNELS];
} DST_ARGS;

/* Macros -------------------------------------------------------------*/

/* Prototypes ---------------------------------------------------------*/
VOID DSTSetDefaults(DST_ARGS * args);
BOOL DSTEncodeFrame(DST_FRAME * frame, UINT8 order, VOID *ptr, size_t *n);
BOOL DSTDecodeFrame(DST_FRAME * frame, UINT8 order, VOID *ptr, size_t *n);


#endif
