/* $Id: ascii.c,v 1.6 2003/03/22 21:16:35 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	ASCII output...

-----------------------------------------------------------------------*/

#include "ascii.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/
static BOOL ascii_hdrs = TRUE;
static BOOL ascii_simple = FALSE;
static BOOL ascii_volts = FALSE;
static CHAR name_format[MAX_PATH_LEN + 1];

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static BOOL OutputASCII(RECORD * record, CHAR *net_name);
static BOOL WriteASCIIHeader(FILE *file, RECORD * record);
static BOOL WriteASCIIData(FILE *file, RECORD * record);

/*---------------------------------------------------------------------*/
VOID ASCIIIncludeHeaders(BOOL b)
{
	ascii_hdrs = b;
	return;
}

/*---------------------------------------------------------------------*/
VOID ASCIISimpleFormat(BOOL b)
{
	ascii_simple = b;
	return;
}

/*---------------------------------------------------------------------*/
VOID ASCIIVolts(BOOL b)
{
	ascii_volts = b;
	return;
}

/*---------------------------------------------------------------------*/
BOOL IsASCIIVolts(VOID)
{
	return ascii_volts;
}

/*---------------------------------------------------------------------*/
VOID ASCIINameFormat(CHAR *format)
{
	strcpy(name_format, format);
	return;
}

/*---------------------------------------------------------------------*/
BOOL ASCIIConvert(RECORD * head, CHAR *net_name)
{
	RECORD *record;

	ASSERT(head != NULL);

	/* Output an ASCII file per channel */
	for (record = head; record != NULL; record = record->next) {
		if (net_name != NULL) {
			if (strcmp(record->name.network, net_name) != 0)
				continue;
		}
		if (!OutputASCII(record, net_name))
			return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL OutputASCII(RECORD * record, CHAR *net_name)
{
	CHAR filename[MAX_PATH_LEN + 1];
	FILE *file;

	ASSERT(record != NULL);

	OutputFilename(record, net_name, name_format, filename);
	LogMessage(LOG_INFO, "%s\n", filename);
	if ((file = fopen(filename, "w+t")) == NULL) {
		LogMessage(LOG_ERR, "ERROR: OutputASCII: fopen(%s) failed: %s\n", filename, strerror(errno));
		return FALSE;
	}

	if (ascii_hdrs) {
		if (!WriteASCIIHeader(file, record)) {
			fclose(file);
			return FALSE;
		}
	}

	if (!WriteASCIIData(file, record)) {
		fclose(file);
		return FALSE;
	}

	fclose(file);

	AppendFilenameToList(filename);

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteASCIIHeader(FILE *file, RECORD * record)
{
	CHAR string[32];
    INT32 year, doy, hour, minute;
    REAL64 second;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

    if (ascii_simple) {
	    fprintf(file, "%u\n", record->unit);
	    fprintf(file, "%u\n", record->channel);
	    fprintf(file, "%s:%s:%s\n", record->name.network, record->name.station, 
                record->name.component);
        DecodeUSTime(record->ist, &year, &doy, &hour, &minute, &second);
        fprintf(file, "%04d\n%03d\n%02d\n%02d\n%09.6lf\n", 
                year, doy, hour, minute, second);
        fprintf(file, "%.6lf\n", 1.0 / record->rate);
        fprintf(file, "%d\n", record->counts_per_volt);
        fprintf(file, "%u\n", record->n);
    } else {
	    fprintf(file, "# GRF unit number:     %u\n", record->unit);
	    fprintf(file, "# GRF channel number:  %u\n", record->channel);
	    fprintf(file, "# Network ID:          %s\n", record->name.network);
	    fprintf(file, "# Station ID:          %s\n", record->name.station);
	    fprintf(file, "# Component ID:        %s\n", record->name.component);
	    fprintf(file, "# Initial sample time: %s %s\n",
		        FormatUSTime(string, 32, UST_DATE_TIME_USEC, record->ist), 
                GRFDescribeQuality(record->timing.time_quality));
	    fprintf(file, "# Sampling rate:       %.6lf %s\n", record->rate, 
                GRFDescribeQuality(record->timing.rate_quality));
	    fprintf(file, "# Counts per volt:     %d\n", record->counts_per_volt);
	    fprintf(file, "# Number of samples:   %u\n", record->n);
    }

	if (ferror(file))
		return FALSE;

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteASCIIData(FILE *file, RECORD * record)
{
	UINT32 i, columns;
	INT32 *sample;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	sample = record->samples;

	if (ascii_volts) {
        columns = (ascii_simple ? 1 : 5);
		for (i = 0; i < record->n; i++) {
			fprintf(file, "%+le ", (REAL64)(*sample++) / (REAL64)record->counts_per_volt);
			if ((i + 1) % columns == 0)
				fprintf(file, "\n");
		}
	}
	else {
        if (ascii_simple) {
		    for (i = 0; i < record->n; i++) {
			    fprintf(file, "%+d\n", *sample++);
		    }
        } else {
		    for (i = 0; i < record->n; i++) {
			    fprintf(file, "%9d", *sample++);
			    if ((i + 1) % 8 == 0)
				    fprintf(file, "\n");
		    }
        }
	}

	if (ferror(file))
		return FALSE;

	return TRUE;
}
