/* $Id: config.c,v 1.13 2003/05/02 00:06:30 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000-2003 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

    Command line parser...

-----------------------------------------------------------------------*/

#include "config.h"

#include "ascii.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static VOID CmdLineHelp(VOID);

/*---------------------------------------------------------------------*/
#include "getopt.c"
BOOL ParseCommandLine(int argc, char *argv[], ARGS * args)
{
    int i, option;

    ASSERT(args != NULL);
    ASSERT(argv != NULL);

    /* Don't write errors to stderr */
    my_opterr = 0;

    /* Options... */
    while ((option = my_getopt(argc, argv, "hxvSqdlbNn:s:f:r:t:o:L:")) != -1) {
        switch (option) {
          case 'h':
            CmdLineHelp();
            break;
          case 'x':
            args->drop_bad_times = TRUE;
            break;
          case 'v':
            ASCIIVolts(TRUE);
            break;
          case 'S':
            ASCIISimpleFormat(TRUE);
            break;
          case 'q':
            SetLogLevel(LOG_WARNING);
            break;
          case 'd':
            SetLogLevel(LOG_DEBUG);
            break;
          case 'b':
            SetByteOrder(ENDIAN_BIG);
            break;
          case 'l':
            SetByteOrder(ENDIAN_LITTLE);
            break;
          case 'N':
            ASCIIIncludeHeaders(FALSE);
            break;
          case 'n':
            if (my_optarg)
                strncpy(args->name_format, my_optarg, MAX_PATH_LEN);
            break;
          case 'o':
            if (my_optarg) {
                strncpy(args->output_path, my_optarg, MAX_PATH_LEN);
                if (args->output_path[strlen(args->output_path) - 1] != '/' &&
                    args->output_path[strlen(args->output_path) - 1] != '\\') {
                    strcat(args->output_path, "/");
                }
            }
            break;
          case 'L':
            if (my_optarg) 
                strncpy(args->list_file, my_optarg, MAX_PATH_LEN);
            break;
          case 'r':
            if (my_optarg)
                args->length = (USTIME) strtoul(my_optarg, NULL, 0) * UST_SECOND;
            break;
          case 't':
            if (my_optarg)
                args->timeout = (USTIME) strtoul(my_optarg, NULL, 0) * UST_SECOND;
            break;
          case 's':
            if (my_optarg) {
                args->src.type = SOURCE_SOCKET;
                strncpy(args->src.name, my_optarg, MAX_PATH_LEN);
                if (!ParseEndpoint(&args->src.des.sock.endpoint, my_optarg, DFL_PORT))
                    return FALSE;
            }
            break;
          case 'f':
            if (my_optarg) {
                if (strncasecmp(my_optarg, "ASCII", 2) == 0)
                    args->format = FMT_ASCII;
                else if (strncasecmp(my_optarg, "GSE2.0", 2) == 0)
                    args->format = FMT_GSE;
                else if (strncasecmp(my_optarg, "SAC", 2) == 0)
                    args->format = FMT_SAC;
                else if (strncasecmp(my_optarg, "SEG-Y", 3) == 0)
                    args->format = FMT_SEGY;
                else if (strncasecmp(my_optarg, "SEED", 3) == 0)
                    args->format = FMT_SEED;
                else if (strncasecmp(my_optarg, "SUDS", 2) == 0)
                    args->format = FMT_SUDS;
                else if (strncasecmp(my_optarg, "MATLAB", 2) == 0)
                    args->format = FMT_MATLAB;
                else if (strncasecmp(my_optarg, "STATS", 2) == 0)
                    args->format = FMT_STATS;
                else
                    fprintf(stderr, "WARNING: unknown output format: -f %s\n", my_optopt);
            }
            break;
          case '?':
            if (isprint(my_optopt))
                fprintf(stderr, "Unknown option `-%c'.\n", my_optopt);
            else
                fprintf(stderr, "Unknown option character `\\x%x'.\n", my_optopt);
            return FALSE;
          default:
            abort();
        }
    }

    /* Arguments... */
    for (i = my_optind; i < argc; i++) {
        if (args->src.type == SOURCE_NONE) {
            args->src.type = SOURCE_FILE;
            strcpy(args->src.name, argv[i]);
        }
        else {
            fprintf(stderr, "unrecognized argument: `%s'\n", argv[i]);
            return FALSE;
        }
    }

    if (args->src.type == SOURCE_NONE)
        CmdLineHelp();

    return TRUE;
}

/*---------------------------------------------------------------------*/
VOID SetDefaults(ARGS * args)
{

    ASSERT(args != NULL);

    memset(args, 0, sizeof(ARGS));

    args->src.type = SOURCE_NONE;
    args->format = DFL_FORMAT;
    args->length = DFL_LENGTH;
    args->name_format[0] = '\0';
    strcpy(args->output_path, "./");
    args->timeout = GRF_DFL_TIMEOUT;
	args->drop_bad_times = FALSE;
	args->list_file[0] = '\0';

    SetLogLevel(DFL_LOGLEVEL);

    return;
}

/*---------------------------------------------------------------------*/
VOID DumpConfiguration(ARGS * args)
{
    CHAR string[65];

    ASSERT(args != NULL);

    LogMessage(LOG_INFO, "GRF source:\n");
    LogMessage(LOG_INFO, " Type:         %s\n", (args->src.type == SOURCE_FILE ? "File" : "Socket"));
    LogMessage(LOG_INFO, " Name:         %s\n", args->src.name);

    if (args->format == FMT_ASCII) {
        LogMessage(LOG_INFO, "Output format: ASCII\n");
    } else if (args->format == FMT_GSE) {
        LogMessage(LOG_INFO, "Output format: GSE2.0\n");
    } else if (args->format == FMT_SAC) {
        LogMessage(LOG_INFO, "Output format: SAC\n");
	    LogMessage(LOG_INFO, "Byte order:    %s endian\n", (GetByteOrder() == ENDIAN_BIG ? "Big" : "Little"));
    } else if (args->format == FMT_SEGY) {
        LogMessage(LOG_INFO, "Output format: PASSCAL modified SEG-Y\n");
	    LogMessage(LOG_INFO, "Byte order:    %s endian\n", (GetByteOrder() == ENDIAN_BIG ? "Big" : "Little"));
    } else if (args->format == FMT_SEED) {
        LogMessage(LOG_INFO, "Output format: Data Only SEED Volumes (Mini-SEED)\n");
	    LogMessage(LOG_INFO, "Byte order:    %s endian\n", (GetByteOrder() == ENDIAN_BIG ? "Big" : "Little"));
    } else if (args->format == FMT_SUDS) {
        LogMessage(LOG_INFO, "Output format: PC-SUDS 1.51\n");
    } else if (args->format == FMT_MATLAB) {
        LogMessage(LOG_INFO, "Output format: MatLab 'Version 4 MAT-File Format'\n");
	    LogMessage(LOG_INFO, "Byte order:    %s endian\n", (GetByteOrder() == ENDIAN_BIG ? "Big" : "Little"));
	} else if (args->format == FMT_STATS) {
        LogMessage(LOG_INFO, "Output format: Computing per channel data statistics\n");
	}

    LogMessage(LOG_INFO, "Output path:   %s\n", args->output_path);
    LogMessage(LOG_INFO, "Record length: %s\n", FormatUSTimerInterval(string, 64, args->length, UST_VERBOSE));

	if (args->list_file[0] != '\0')
		LogMessage(LOG_INFO, "Filename list: %s\n", args->list_file);

	if (args->drop_bad_times)
	    LogMessage(LOG_INFO, "Dropping packets with 'unknown' or 'bad' time quality!\n");

	LogMessage(LOG_INFO, "\n");

    return;
}

/*---------------------------------------------------------------------*/
static VOID CmdLineHelp(VOID)
{

    LogMessage(LOG_NOTICE, "Usage: grfconvert [-hqdblNSvx] [-n name_format] [-f format] [-r rec_len]\n");
    LogMessage(LOG_NOTICE, "                  [-t seconds] [-L list_file] [-o path] -s socket|image_file\n");
    LogMessage(LOG_NOTICE, "\n");
    LogMessage(LOG_NOTICE, "   -h           Help display.\n");
    LogMessage(LOG_NOTICE, "   -q           Quiet logging.\n");
    LogMessage(LOG_NOTICE, "   -d           Debug logging.\n");
    LogMessage(LOG_NOTICE, "   -b           Big endian output, if appropriate. (native) \n");
    LogMessage(LOG_NOTICE, "   -l           Little endian output, if appropriate. (native) \n");
    LogMessage(LOG_NOTICE, "   -N           If format is ASCII, don't include header info.\n");
    LogMessage(LOG_NOTICE, "   -S           If format is ASCII, use simple single column format.\n");
    LogMessage(LOG_NOTICE, "   -v           If format is ASCII or STATS, output samples in volts.\n");
    LogMessage(LOG_NOTICE, "   -x           Drop data packets with 'unknown' or 'bad' time quality.\n");
    LogMessage(LOG_NOTICE, "   -n format    Output file naming format string.\n");
    LogMessage(LOG_NOTICE, "   -f format    Output data format:\n");
    LogMessage(LOG_NOTICE, "                   ASCII, (GSE2.0), SAC, SEG-Y, SEED, SUDS, MATLAB, or STATS.\n");
    LogMessage(LOG_NOTICE, "   -r rec_len   Output record length in seconds. (%u seconds)\n",
        (UINT32)(DFL_LENGTH / UST_SECOND));
    LogMessage(LOG_NOTICE, "   -t seconds   Set socket read timeout. (%u seconds)\n", (UINT32)(DFL_TIMEOUT / UST_SECOND));
    LogMessage(LOG_NOTICE, "   -L list_file ASCII list file to append output filenames to. (none)\n");
    LogMessage(LOG_NOTICE, "   -o path      Output directory. (./)\n");
    LogMessage(LOG_NOTICE, "   -s socket    Socket endpoint to read GRF packets from.\n");
	LogMessage(LOG_NOTICE, "                port number defaults to %u if not specified.\n", DFL_PORT);
    LogMessage(LOG_NOTICE, "   image_file   GRF packet image file to read packets from.\n");
    LogMessage(LOG_NOTICE, "\n");
    LogMessage(LOG_NOTICE, "[] = optional, () = default, | = mutually exclusive.\n");

    exit(1);
}
