/* $Id: matlab.c,v 1.4 2003/03/22 21:16:35 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	MatLab binary output...

-----------------------------------------------------------------------*/

#include "matlab.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* MatLab file header... */ 
typedef struct TAG_FMATRIX {
	INT32 type;
	INT32 rows;
	INT32 columns;
	INT32 imag_flag;
	INT32 name_len;
} FMATRIX;

/* Module globals -----------------------------------------------------*/
static CHAR name_format[MAX_PATH_LEN + 1];

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static BOOL OutputMatLab(RECORD * record, CHAR *net_name);
static BOOL WriteMatLabHeader(FILE *file, RECORD * record);
static BOOL WriteMatLabData(FILE *file, RECORD * record);

/*---------------------------------------------------------------------*/
VOID MatLabNameFormat(CHAR *format)
{
	strcpy(name_format, format);
	return;
}

/*---------------------------------------------------------------------*/
BOOL MatLabConvert(RECORD * head, CHAR *net_name)
{
	RECORD *record;

	ASSERT(head != NULL);

	/* Output a MatLab file per channel */
	for (record = head; record != NULL; record = record->next) {
		if (net_name != NULL) {
			if (strcmp(record->name.network, net_name) != 0)
				continue;
		}
		if (!OutputMatLab(record, net_name))
			return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL OutputMatLab(RECORD * record, CHAR *net_name)
{
	CHAR filename[MAX_PATH_LEN + 1];
	FILE *file;

	ASSERT(record != NULL);

	OutputFilename(record, net_name, name_format, filename);
	LogMessage(LOG_INFO, "%s\n", filename);
	if ((file = fopen(filename, "w+b")) == NULL) {
		LogMessage(LOG_ERR, "ERROR: OutputMatLab: fopen(%s) failed: %s\n", filename, strerror(errno));
		return FALSE;
	}

	if (!WriteMatLabHeader(file, record)) {
		fclose(file);
		return FALSE;
	}

	if (!WriteMatLabData(file, record)) {
		fclose(file);
		return FALSE;
	}

	fclose(file);

	AppendFilenameToList(filename);

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteMatLabHeader(FILE *file, RECORD * record)
{
	CHAR name[32];
    INT32 type, name_len;
	REAL64 real;
	FMATRIX hdr;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	/* Write a variable with the time */
	sprintf(name, "%s_%s_%s_time", record->name.network, record->name.station, record->name.component);
    if(GetByteOrder() == ENDIAN_BIG)
        type = 1000;
    else
        type = 0;
    hdr.type = SwapINT32(type);
	hdr.rows = SwapINT32(1);
	hdr.columns = SwapINT32(1);
	hdr.imag_flag = 0;
	name_len = (INT32)strlen(name) + 1;
    hdr.name_len = SwapINT32(name_len);
	/* Write header followed by name to file */
	if (fwrite(&hdr, sizeof(FMATRIX), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	if (fwrite(name, name_len, 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	/* Write time as real seconds since epoch (1/1/1970 00:00 UTC) */
	real = (REAL64)record->ist / (REAL64)UST_SECOND;
	if (fwrite(&real, sizeof(REAL64), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	/* Write a variable with the sampling period */
	sprintf(name, "%s_%s_%s_delta", record->name.network, record->name.station, record->name.component);
    if(GetByteOrder() == ENDIAN_BIG)
        type = 1000;
    else
        type = 0;
    hdr.type = SwapINT32(type);
	hdr.rows = SwapINT32(1);
	hdr.columns = SwapINT32(1);
	hdr.imag_flag = 0;
	name_len = (INT32)strlen(name) + 1;
    hdr.name_len = SwapINT32(name_len);
	/* Write header followed by name to file */
	if (fwrite(&hdr, sizeof(FMATRIX), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	if (fwrite(name, name_len, 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	/* Write sampling period */
	real = 1.0 / (REAL64)record->rate;
	if (fwrite(&real, sizeof(REAL64), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	/* Write a variable with counts per volt */
	sprintf(name, "%s_%s_%s_volt", record->name.network, record->name.station, record->name.component);
    if(GetByteOrder() == ENDIAN_BIG)
        type = 1000;
    else
        type = 0;
    hdr.type = SwapINT32(type);
	hdr.rows = SwapINT32(1);
	hdr.columns = SwapINT32(1);
	hdr.imag_flag = 0;
	name_len = (INT32)strlen(name) + 1;
    hdr.name_len = SwapINT32(name_len);
	/* Write header followed by name to file */
	if (fwrite(&hdr, sizeof(FMATRIX), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	if (fwrite(name, name_len, 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	/* Write counts per volt */
	real = (REAL64)record->counts_per_volt;
	if (fwrite(&real, sizeof(REAL64), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	/* Write vector header for the data... */
	sprintf(name, "%s_%s_%s", record->name.network, record->name.station, record->name.component);
    if(GetByteOrder() == ENDIAN_BIG)
        type = 1000;
    else
        type = 0;
    hdr.type = SwapINT32(type);
	hdr.rows = SwapINT32(record->n);
	hdr.columns = SwapINT32(1);
	hdr.imag_flag = 0;
	name_len = (INT32)strlen(name) + 1;
    hdr.name_len = SwapINT32(name_len);
	/* Write header followed by name to file */
	if (fwrite(&hdr, sizeof(FMATRIX), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}
	if (fwrite(name, name_len, 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteMatLabData(FILE *file, RECORD * record)
{
	UINT32 i;
	INT32 *sample;
	REAL64 real;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	/* Write data as 64 bit IEEE reals */
	sample = record->samples;
	for (i = 0; i < record->n; i++) {
		real = (REAL64)SwapINT32(sample[i]);
		if (fwrite(&real, sizeof(REAL64), 1, file) != 1) {
			LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
			return FALSE;
		}
	}

	return TRUE;
}
