/* $Id: seed.h,v 1.2 2001/09/25 01:12:10 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2001 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

    SEED types...

-----------------------------------------------------------------------*/
#if !defined _SEED_H_INCLUDED_
#define _SEED_H_INCLUDED_

#include "main.h"
#include "seedtime.h"
#include "seeddata.h"

#define SEEDVER 22                          /* Version 2.0 of the SEED format */

typedef REAL64 VOLFLT;

typedef struct _blockette {
    UINT8 *BlkBuff;                         /* Pointer to the current blockette */
    UINT32 BufLen;                          /* Max Length of the buffer */
    UINT16 BlkType;                         /* The blockette id number */
    UINT16 BlkVer;                          /* The version of the blockette */
    UINT8 BlkRec;                           /* Record type of blockette */
    UINT32 PackPos;                         /* Number of bytes this blockette */
    UINT16 ParsePos;                        /* Where is the parser now? */
} BLOCKETTE;

typedef struct _logical_record {
    UINT8 *RecBuff;                         /* Pointer to current record */
    UINT32 RecNum;                          /* Record number of current record */
    UINT16 NextPos;                         /* Next position to write to */
    UINT16 RecLen;                          /* logical rec len power of 2 exp */
    UINT16 RecSize;                         /* The size of the record */
    BOOL Continue;                          /* Is this a continuation record */
    UINT8 RecType;                          /* What is the control header type */
     BOOL(*WriteService) ();                /* Subroutine to dump buffers */
     BOOL(*ReadService) ();                 /* Routine to read next */
    UINT32 BlockExp;                        /* Exponent for block buffer */
    UINT32 BlockMax;                        /* The number of bytes of buffer */
    UINT32 BlockSize;                       /* Size of last read */
    UINT32 BlockPos;                        /* Current position */
    UINT8 *BlockBuffer;                     /* Pointer to buffer */
} LOGICAL_RECORD;

typedef struct _abbrev {
    UINT8 *Abbrev_Key;                      /* How do we access this? */
    UINT16 Abbrev_ID;                       /* Numeric id for writing headers */
    UINT8 *Comment;                         /* Text for abbreviation */
    UINT16 Use_Count;                       /* How many times refered to */
    struct _abbrev *Next;
} ABBREV;

typedef struct _unit {
    UINT8 *Unit_Key;                        /* Key for the units */
    UINT16 Unit_ID;                         /* What code units? */
    UINT8 *Comment;                         /* Description of Unit */
    UINT16 Use_Count;                       /* Number of times refered to */
    struct _unit *Next;
} UNIT;

typedef struct _comment {
    UINT16 Comment_ID;                      /* Comment numeric ident */
    UINT8 Class;                            /* Class Code */
    UINT8 *Text;                            /* Text of comment */
    UNIT *Level;                            /* The unit of the level */
    UINT16 Use_Count;                       /* Number of times refered to */
    struct _comment *Next;
} COMMENT;

#define MAXKEYS 10
typedef struct _format {
    UINT8 *Format_Key;                      /* What is name of format */
    UINT8 *Format_Name;                     /* Description of format */
    UINT16 Format_ID;                       /* Id of format */
    UINT16 Family;
    UINT8 NumKeys;
    UINT8 *Keys[MAXKEYS];
    UINT16 Use_Count;                       /* Number of times refered to */
    struct _format *Next;
} FORMAT;

typedef struct _comment_entry {
    STDTIME Start_Comment;
    STDTIME End_Comment;
    COMMENT *Comment;
    LONG Comment_Level;
    struct _comment_entry *Next;
} COMMENT_ENTRY;

typedef struct _zeros_poles {
    UINT8 *Citation_Name;
    UINT8 *Title;
    UINT8 Function_Type;
    UINT16 Cascade;
    UNIT *Response_In;
    UNIT *Response_Out;
    VOLFLT AO_Normalization;
    VOLFLT Norm_Frequency;
    UINT16 Complex_Zeros;
    VOLFLT *Zero_Real_List;
    VOLFLT *Zero_Imag_List;
    VOLFLT *Zero_Real_Errors;
    VOLFLT *Zero_Imag_Errors;
    UINT16 Complex_Poles;
    VOLFLT *Pole_Real_List;
    VOLFLT *Pole_Imag_List;
    VOLFLT *Pole_Real_Errors;
    VOLFLT *Pole_Imag_Errors;
} ZEROS_POLES;

typedef struct _coefficients {
    UINT8 *Citation_Name;
    UINT8 *Title;
    UINT8 Function_Type;
    UINT16 Cascade;
    UNIT *Response_In;
    UNIT *Response_Out;
    UINT16 Numerators;
    VOLFLT *Numer_Coeff;
    VOLFLT *Numer_Error;
    UINT16 Denominators;
    VOLFLT *Denom_Coeff;
    VOLFLT *Denom_Error;
} COEFFICIENTS;

typedef struct _resp_generic {
    UINT8 *Citation_Name;
    UINT8 *Title;
    UINT8 Function_Type;
    UINT16 Cascade;
    UNIT *Response_In;
    UNIT *Response_Out;
    UINT16 Corners;
    VOLFLT *Corner_Freq;
    VOLFLT *Corner_Slope;
} RESP_GENERIC;

typedef struct _resp_list {
    UINT8 *Citation_Name;
    UINT8 *Title;
    UINT16 Cascade;
    UNIT *Response_In;
    UNIT *Response_Out;
    UINT16 Number_Responses;
    VOLFLT *Frequency;
    VOLFLT *Amplitude;
    VOLFLT *Amplitude_Error;
    VOLFLT *Phase_Angle;
    VOLFLT *Phase_Error;
} RESP_LIST;

typedef struct _decimation {
    UINT8 *Citation_Name;
    UINT8 *Title;
    UINT16 Cascade;
    VOLFLT Input_Rate;                      /* Input sample rate (sps) */
    LONG Factor;                            /* Decimation Factor */
    LONG Offset;                            /* Decimation Offset */
    VOLFLT Delay;                           /* Estimated Delay (secs) */
    VOLFLT Correction;                      /* Correction Applied (secs) */
} DECIMATION;

typedef struct _sensitivity {
    UINT16 Cascade;                         /* Zero if final capping sens */
    VOLFLT Sensitivity;                     /* Accumulated average sens */
    VOLFLT Frequency;                       /* Frequency of sens */
    UINT8 Num_Cals;                         /* Number of calibrations */
    VOLFLT *Cal_Value;                      /* Value of cal */
    VOLFLT *Cal_Freq;                       /* Frequency of cal */
    STDTIME *Cal_Time;                      /* Time of above cal */
} SENSITIVITY;

typedef struct _response_ptr {
    union {
        ZEROS_POLES *PZ;
        COEFFICIENTS *CO;
        RESP_LIST *RL;
        RESP_GENERIC *GR;
        DECIMATION *DM;
        SENSITIVITY *SENS;
    } ptr;
    UINT8 type;
    struct _response_ptr *Next;
} RESPONSE;

#define RESP_ZEROS_POLES  'P'
#define RESP_COEFFICIENTS 'C'
#define RESP_RESP_LIST    'L'
#define RESP_GENERIC      'G'
#define RESP_SENSITIVITY  'S'
#define RESP_DECIMATION   'D'

typedef struct _channel_entry {
    UINT8 *Location;
    UINT8 *Identifier;
    UINT16 SubChannel;
    ABBREV *Instrument;
    UINT8 *Optional;
    UNIT *Signal_Response;
    UNIT *Calibration_Input;
    VOLFLT Lattitude;
    VOLFLT Longitude;
    VOLFLT Elevation;
    VOLFLT Local_Depth;
    VOLFLT Azimuth;
    VOLFLT Dip;
    FORMAT *Format_Type;
    UINT8 Data_Exp;
    VOLFLT Sample_Rate;
    VOLFLT Max_Drift;
    UINT32 Channel_Flags;
#define CFG_TRIGGERED   0x00000001          /* Data is triggered */
#define CFG_CONTINUOUS  0x00000002          /* Data recorded continuously */
#define CFG_HEALTHCHAN  0x00000004          /* State of health channel */
#define CFG_GEODATA     0x00000008          /* Geophysical data */
#define CFG_ENVIRON     0x00000010          /* Weather/Environmental data */
#define CFG_FLAGS       0x00000020          /* Flags/Switch information */
#define CFG_SYNTH       0x00000040          /* Data is synthesized */
#define CFG_CALIN       0x00000080          /* Channel is a cal input signal */
#define CFG_EXPERIMENT  0x00000100          /* Channel is experimental */
#define CFG_MAINTENANCE 0x00000200          /* Maintenance underway on channel */
#define CFG_BEAM        0x00000400          /* Data is a beam synthesis */
    UINT16 Number_Comments;
    COMMENT_ENTRY *Root_Comment;
    COMMENT_ENTRY *Tail_Comment;
    STDTIME Effective_Start;
    STDTIME Effective_End;
    UINT8 Update_Flag;
    struct _channel_entry *Update_Channel;
    struct _channel_entry *Root_Channel;    /* First channel of this set */
    RESPONSE *Root_Response;
    RESPONSE *Tail_Response;
    struct _channel_entry *Next;
    VOID *Root_Data;                        /* Link to user database */
    VOID *Tail_Data;                        /* Link to user database */
    UINT32 Use_Count;
} CHANNEL_ENTRY;

typedef struct _station_entry {
    UINT32 *RecNum;                         /* Where was this written */
    UINT8 *Call_Letters;
    UINT16 Station_Number;
    VOLFLT Lattitude;
    VOLFLT Longitude;
    VOLFLT Elevation;
    UINT8 *Site_Name;
    ABBREV *Station_Owner;
    UINT16 Long_Order;
    UINT16 Word_Order;
    UINT16 Number_Channels;
    UINT16 Number_Comments;
    CHANNEL_ENTRY *Root_Channel;
    CHANNEL_ENTRY *Tail_Channel;
    COMMENT_ENTRY *Root_Comment;
    COMMENT_ENTRY *Tail_Comment;
    STDTIME Effective_Start;
    STDTIME Effective_End;
    UINT8 Update_Flag;
    struct _station_entry *Update_Station;
    struct _station_entry *Root_Station;
    struct _station_entry *Next;
    UINT32 Use_Count;
    STDTIME Established;
    VOID *Root_Data;                        /* Link to user database */
    VOID *Tail_Data;                        /* Link to user database */
} STATION_ENTRY;

typedef struct _volume_id {
    UINT8 LenExp;                           /* Record length exponent */
    STDTIME Vol_Begin;                      /* Beginning time of volume */
    STDTIME End_Volume;                     /* End time of volume */
    ABBREV *Root_Abbrev;
    ABBREV *Tail_Abbrev;
    UNIT *Root_Unit;
    UNIT *Tail_Unit;
    FORMAT *Root_Format;
    FORMAT *Tail_Format;
    COMMENT *Root_Comment;
    COMMENT *Tail_Comment;
    STATION_ENTRY *Root_Station;
    STATION_ENTRY *Tail_Station;
    STDTIME Effective_Start;
    STDTIME Effective_End;
} VOLUME_ID;

// Blockette types ----------------------------------------------------

#define SEED_STATION_HEADER   5
#define SEED_NETWORK_HEADER   10
#define SEED_STATION_INDEX    11
#define SEED_SPAN_INDEX       12
#define SEED_FORMAT_DICT      30
#define SEED_COMMENT_DICT     31
#define SEED_SOURCE_DICT      32
#define SEED_ABBREV_DICT      33
#define SEED_UNIT_DICT        34
#define SEED_BEAM_DICT        35
#define SEED_STATION_RECORD   50
#define SEED_STATION_COMMENT  51
#define SEED_CHANNEL_RECORD   52
#define SEED_ZEROS_POLES      53
#define SEED_COEFFICIENTS     54
#define SEED_RESPONSE_LIST    55
#define SEED_GENERIC_RESPONSE 56
#define SEED_DECIMATION       57
#define SEED_GAIN_SENSITIVITY 58
#define SEED_CHANNEL_COMMENT  59
#define SEED_TIME_SPAN_HEADER 70
#define SEED_HYPOCENTER       71
#define SEED_EVENT_PHASES     72
#define SEED_TIME_SPAN_INDEX  73

#endif
