/* $Id: seeddata.h,v 1.1 2001/09/24 18:43:50 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2001 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

    SEED data types...

-----------------------------------------------------------------------*/
#if !defined _SEEDDATA_H_INCLUDED_
#define _SEEDDATA_H_INCLUDED_

#include "main.h"

typedef struct net_time {
    UINT16 year;
    UINT16 day;
    UINT8 hour;
    UINT8 minute;
    UINT8 seconds;
    UINT8 dummy;
    UINT16 fracs;                           /* Number of .0001 seconds */
} NETTIME;

typedef struct net_float {
    INT16 mantissa;
    INT16 exponent;
} NETFLT;

/*
 *	Fixed portion of the data block
 */

typedef struct _fixed_block {
    UINT8 Seq_ID[6];                        /* Sequence number of record */
    UINT8 Record_Type;                      /* Always a 'D' */
    UINT8 Filler;                           /* Always a space */
    UINT8 Station_ID[5];                    /* Station id (space filled) */
    UINT8 Location_ID[2];                   /* Array/Extended Station (filled) */
    UINT8 Channel_ID[3];                    /* Channel Id (space filled) */
    UINT8 Network_ID[2];                    /* Extended Network Type */
    NETTIME Start_Time;                     /* Start time of record */
    UINT16 Number_Samps;                    /* Number of samples in record */
    INT16 Rate_Factor;                      /* Sample rate factor */
    INT16 Rate_Mult;                        /* Rate Multiplier */
    UINT8 Activity_Flags;                   /* Activity Information */
    UINT8 IO_Flags;                         /* I/O Information */
    UINT8 Qual_Flags;                       /* Data Quality Information */
    UINT8 Total_Blockettes;                 /* Number blockettes to follow */
    INT32 Time_Correction;                  /* Number of .0001 sec correction */
    UINT16 Data_Start;                      /* Byte where data starts */
    UINT16 First_Blockette;                 /* Byte of first blockette */
} SEED_DATA;

#define ACTFLAG_CALSIG 0x01                 /* Calibration signals this record */
#define ACTFLAG_CLKFIX 0x02                 /* Error caused by clock correction */
#define ACTFLAG_BEGEVT 0x04                 /* Beginning of event */
#define ACTFLAG_ENDEVT 0x08                 /* End of event */
#define IOFLAG_ORGPAR  0x01                 /* Original tape had parity error */
#define IOFLAG_LONGRC  0x02                 /* Original read long record */
#define IOFLAG_SHORTR  0x04                 /* Original had short record */
#define QULFLAG_AMPSAT 0x01                 /* Amplifier saturation detected */
#define QULFLAG_SIGCLP 0x02                 /* Signal clipping detected */
#define QULFLAG_SPIKES 0x04                 /* Signal spiking detected */
#define QULFLAG_GLITCH 0x08                 /* Signal glitch detected */
#define QULFLAG_PADDED 0x10                 /* Data missing or padded */
#define QULFLAG_TLMSNC 0x20                 /* Telemetry sync error/dropout */
#define QULFLAG_CHARGE 0x40                 /* Digitial filter charging */
#define QULFLAG_TIMERR 0x80                 /* Time tag questionable */

/*
 *	Generic Event Detection Blockette [200]
 */

#define BLK_EVTDET 200

struct event_detect {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    NETFLT Signal_Amplitude;                /* Event signal amplitude */
    NETFLT Signal_Period;                   /* Period of signal */
    NETFLT Background_Estimate;             /* Estimation of background */
    UINT8 Event_Flags;                      /* Event detection flags */
    UINT8 Rsrvd[1];                         /* Reserved (must be 0) */
    NETTIME Onset_Time;                     /* Onset Time of Event */
};

#define EVTDET_SZ (sizeof (struct event_detect))
#define EVENT_DILAT 0x01                    /* Set is dilatation, unset is compression */
#define EVENT_DECON 0x02                    /* When set results are deconvolved, if unset results are digital counts */

/*
 *	Murdock Event Detection Blockette [201]
 */

#define BLK_MURDET 201

struct murdock_detect {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    NETFLT Signal_Amplitude;                /* Event signal amplitude */
    NETFLT Signal_Period;                   /* Period of signal */
    NETFLT Background_Estimate;             /* Estimation of background */
    UINT8 Event_Flags;                      /* Event detection flags */
    UINT8 Rsrvd[1];                         /* Reserved (must be 0) */
    NETTIME Onset_Time;                     /* Onset Time of Event */
    UINT8 SNR_Qual[6];                      /* Quality values */
    UINT8 Lookback;                         /* Lookback type */
    UINT8 Pick;                             /* Pick algorithm */
};

#define MURDET_SZ (sizeof (struct murdock_detect))
#define MEVENT_DILAT 0x01                   /* Set is dilatation, unset is compression */

/*
 *	Step Cal Blockette [300]
 */

#define BLK_STEPCAL 300

struct step_cal {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    UINT16 Cal_In_Chan;                     /* Channel ID containing cal input zero denotes none */
    NETTIME First_Cal;                      /* Time of first cal in sequence (might be in previous records) */
    UINT32 Duration;                        /* Number of .1ms step duration */
    UINT32 Interval;                        /* Number of .1ms tween step on times */
    NETFLT Amplitude;                       /* Amp of signal in units */
    UINT8 Number_Steps;                     /* Number of cal steps in sequence */
    UINT8 Cal_Flags;                        /* Cal information */
    UINT8 Reserved[2];
};

#define STEPCAL_SZ (sizeof (struct step_cal))
#define STEP_POSITI 0x01                    /* First step was positive */
#define STEP_ALTERN 0x02                    /* Steps alternate sign */
#define STEP_AUTOMA 0x04                    /* Cal was automatic, otherwise manual */
#define STEP_CONTIN 0x08                    /* Cal begin in a previous record */

/*
 *	Sine Wave Cal Blockette [310]
 */

#define BLK_SINECAL 310

struct sine_cal {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    UINT16 Cal_In_Chan;                     /* Channel ID containing cal input zero denotes none */
    NETTIME Cal_Start;                      /* Time when cal begins (might be in previous records) */
    UINT32 Duration;                        /* Number of .1ms cal duration */
    NETFLT Period;                          /* Sine period in seconds */
    NETFLT Amplitude;                       /* Amp of cal sig in units */
    UINT8 Sine_Flags;                       /* Sine cal flags */
    UINT8 Reserved[3];
};

#define SINECAL_SZ (sizeof (struct sine_cal))
#define SINE_AUTOMA 0x04                    /* Cal automatic, otherwise manual */
#define SINE_CONTIN 0x08                    /* Cal began in a previous record */
#define SINE_PKTOPK 0x10                    /* Peak to peak amplitude */
#define SINE_ZRTOPK 0x20                    /* Zero to peak amplitude */
#define SINE_RMSAMP 0x40                    /* RMS amplitude */

/*
 *	Pseudo Random Cal Blockette [320]
 */

#define BLK_RANDOMCAL 320

struct random_cal {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    UINT16 Cal_In_Chan;                     /* Channel ID containing cal input must be present */
    NETTIME Cal_Start;                      /* Time when cal begins (might be in previous records) */
    UINT32 Duration;                        /* Number of .1ms cal duration */
    NETFLT Amplitude;                       /* Amplitude of steps in units */
    UINT8 Random_Flags;                     /* Cal Flags */
    UINT8 Reserved[3];
};

#define RANDOMCAL_SZ (sizeof (struct random_cal))
#define RANDOM_AUTOMA 0x04                  /* Automatic cal, otherwise manual */
#define RANDOM_CONTIN 0x08                  /* Cal began in previous record */
#define RANDOM_RANAMP 0x10                  /* Amps are random too, see cal in chan */

/*
 *	Generic Cal Blockette [390]
 */

#define BLK_GENCALCAL 390

struct Generic_cal {
    UINT16 Blockette_Type;                  /* Blockette identifier */
    UINT16 Next_Begin;                      /* Byte where next blockette begins */
    UINT16 Cal_In_Chan;                     /* Channel ID containing cal input must be present */
    NETTIME Cal_Start;                      /* Time when cal begins (might be in previous records) */
    UINT32 Duration;                        /* Number of .1ms cal duration */
    NETFLT Amplitude;                       /* Amplitude of steps in units */
    UINT8 Gencal_Flags;                     /* Cal Flags */
    UINT8 Reserved[3];
};

#define GENCALCAL_SZ (sizeof (struct Generic_cal))
#define GENCAL_AUTOMA 0x04                  /* Automatic cal, otherwise manual */
#define GENCAL_CONTIN 0x08                  /* Cal began in previous record */

#endif
