/* $Id: segy.c,v 1.3 2003/03/22 21:16:35 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	PASSCAL modified SEG-Y output...

-----------------------------------------------------------------------*/

#include "segy.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* 
 This is the header for the PASSCAL SEGY trace data.

 The PASSCAL SEGY trace format is a modified form of the SEG-Y
 trace format.  The modification comes is because we use some of
 the unspecified header words to store information pertinent to
 the PASSCAL data.  The data values for each trace are preceded
 by a 240 byte header. This format is given below.  All integer
 values are stored with the most significant byte first.  Data
 values are either 16 0r 32 bit integers depending on byte 206
 of the header, the field named "data_form".

 Reading bytes  directly into this header will allow access to
 all of the fields.  The number in the comment is the byte offset
 into the segy file.  An "X" in the comment indicates that field
 is NEVER set.  Fields that are set to default values contain that
 value and follow a ":" in the comment ("grep : segy.h" will spit
 all default fields out).  Two pairs of fields exist to cover an
 inherited limitation; sampleLength/num_samps and deltaSample/samp_rate.
 When the value is too large to fit in a short, sampleLength or
 deltaSample become flags and require their long counterparts,
 num_samps and samp_rate, to contain that value.
*/
typedef struct SegyHead {					/* Offset Description  */
	long lineSeq;							/* 0 Sequence numbers within line */
	long reelSeq;							/* 4 Sequence numbers within reel */
	long event_number;						/* 8 Original field record number or trigger number */
	long channel_number;					/* 12 Trace channel number within the original field record */
	long energySourcePt;					/* 16 X */
	long cdpEns;							/* 20 X */
	long traceInEnsemble;					/* 24 X */
	short traceID;							/* 28 Trace identification code: seismic data = 1 */
	short vertSum;							/* 30 X */
	short horSum;							/* 32 X */
	short dataUse;							/* 34 X */
	long sourceToRecDist;					/* 36 X */
	long recElevation;						/* 40 X */
	long sourceSurfaceElevation;			/* 44 X */
	long sourceDepth;						/* 48 X */
	long datumElevRec;						/* 52 X */
	long datumElemSource;					/* 56 X */
	long sourceWaterDepth;					/* 60 X */
	long recWaterDepth;						/* 64 X */
	short elevationScale;					/* 68 Elevation Scaler: scale = 1 */
	short coordScale;						/* 70 Coordinate Scaler: scale = 1 */
	long sourceLongOrX;						/* 72 X */
	long sourceLatOrY;						/* 76 X */
	long recLongOrX;						/* 80 X */
	long recLatOrY;							/* 84 X */
	short coordUnits;						/* 88 Coordinate Units:  = 2 (Lat/Long) */
	short weatheringVelocity;				/* 90 X */
	short subWeatheringVelocity;			/* 92 X */
	short sourceUpholeTime;					/* 94 X */
	short recUpholeTime;					/* 96 X */
	short sourceStaticCor;					/* 98 X */
	short recStaticCor;						/* 100 X */
	short totalStatic;						/* 102 Total Static in MILLISECS added to Trace Start Time */
	short lagTimeA;							/* 104 X */
	short lagTimeB;							/* 106 X */
	short delay;							/* 108 X */
	short muteStart;						/* 110 X */
	short muteEnd;							/* 112 X */
	short sampleLength;						/* 114 Number of samples in this trace (unless == 32767) */
	short deltaSample;						/* 116 Sampling interval in MICROSECONDS (unless == 1) */
	short gainType;							/* 118 Gain Type: 1 = Fixed Gain */
	short gainConst;						/* 120 Gain of amplifier */
	short initialGain;						/* 122 X */
	short correlated;						/* 124 X */
	short sweepStart;						/* 126 X */
	short sweepEnd;							/* 128 X */
	short sweepLength;						/* 130 X */
	short sweepType;						/* 132 X */
	short sweepTaperAtStart;				/* 134 X */
	short sweepTaperAtEnd;					/* 136 X */
	short taperType;						/* 138 X */
	short aliasFreq;						/* 140 X */
	short aliasSlope;						/* 142 X */
	short notchFreq;						/* 144 X */
	short notchSlope;						/* 146 X */
	short lowCutFreq;						/* 148 X */
	short hiCutFreq;						/* 150 X */
	short lowCutSlope;						/* 152 X */
	short hiCutSlope;						/* 154 X */
	short year;								/* 156 year of Start of trace */
	short day;								/* 158 day of year at Start of trace */
	short hour;								/* 160 hour of day at Start of trace */
	short minute;							/* 162 minute of hour at Start of trace */
	short second;							/* 164 second of minute at Start of trace */
	short timeBasisCode;					/* 166 Time basis code: 2 = GMT */
	short traceWeightingFactor;				/* 168 X */
	short phoneRollPos1;					/* 170 X */
	short phoneFirstTrace;					/* 172 X */
	short phoneLastTrace;					/* 174 X */
	short gapSize;							/* 176 X */
	short taperOvertravel;					/* 178 X */
	char station_name[6];					/* 180 Station Name code (5 chars + \0) */
	char sensor_serial[8];					/* 186 Sensor Serial code (7 chars + \0) */
	char channel_name[4];					/* 194 Channel Name code (3 chars + \0) */
	char extrash[2];						/* 198 Extra unassigned bytes (2 chars) */
	long samp_rate;							/* 200 Sample interval in MICROSECS as a 32 bit integer */
	short data_form;						/* 204 Data Format flag: 0=16 bit, 1=32 bit integer */
	short m_secs;							/* 206 MILLISECONDS of seconds of Start of trace */
	short trigyear;							/* 208 year of Trigger time */
	short trigday;							/* 210 day of year at Trigger time */
	short trighour;							/* 212 hour of day at Trigger time */
	short trigminute;						/* 214 minute of hour at Trigger time */
	short trigsecond;						/* 216 second of minute at Trigger time */
	short trigmills;						/* 218 MILLISECONDS of seconds of Trigger time */
	float scale_fac;						/* 220 Scale Factor (IEEE 32 bit float) volts/count */
	short inst_no;							/* 224 Instrument Serial Number */
	short not_to_be_used;					/* 226 X */
	long num_samps;							/* 228 Number of Samples as a 32 bit integer (when sampleLength == 32767) */
	long max;								/* 232 Maximum value in Counts */
	long min;								/* 236 Minimum value in Counts */
} SEGYHEAD;									/* end of segy trace header */

/* Module globals -----------------------------------------------------*/
static CHAR name_format[MAX_PATH_LEN + 1];

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static BOOL OutputSEGY(RECORD * record, CHAR *net_name);
static BOOL WriteSEGYHeader(FILE *file, RECORD * record);
static BOOL WriteSEGYData(FILE *file, RECORD * record);

/*---------------------------------------------------------------------*/
VOID SEGYNameFormat(CHAR *format)
{
	strcpy(name_format, format);
	return;
}

/*---------------------------------------------------------------------*/
BOOL SEGYConvert(RECORD * head, CHAR *net_name)
{
	RECORD *record;

	ASSERT(head != NULL);

	/* Output an SEGY file per channel */
	for (record = head; record != NULL; record = record->next) {
		if (net_name != NULL) {
			if (strcmp(record->name.network, net_name) != 0)
				continue;
		}
		if (!OutputSEGY(record, net_name))
			return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL OutputSEGY(RECORD * record, CHAR *net_name)
{
	CHAR filename[MAX_PATH_LEN + 1];
	FILE *file;

	ASSERT(record != NULL);

	OutputFilename(record, net_name, name_format, filename);
	LogMessage(LOG_INFO, "%s\n", filename);
	if ((file = fopen(filename, "w+b")) == NULL) {
		LogMessage(LOG_ERR, "ERROR: OutputSEGY: fopen(%s) failed: %s\n", filename, strerror(errno));
		return FALSE;
	}

	if (!WriteSEGYHeader(file, record)) {
		fclose(file);
		return FALSE;
	}

	if (!WriteSEGYData(file, record)) {
		fclose(file);
		return FALSE;
	}

	fclose(file);

	AppendFilenameToList(filename);

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteSEGYHeader(FILE *file, RECORD * record)
{
	INT32 year, doy, hour, minute;
	REAL64 second;
	SEGYHEAD hdr;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	/* Initialize the header */
	memset(&hdr, 0x00, sizeof(SEGYHEAD));

	/* Get needed entries */
	hdr.traceID = SwapINT16(1);
	hdr.channel_number = SwapINT32(record->channel);

	hdr.dataUse = SwapINT16(1);

	hdr.gainType = SwapINT16(1);
	hdr.initialGain = SwapINT16(1);
	hdr.gainConst = SwapINT16(1);

	hdr.scale_fac = SwapREAL32(1.0);

	hdr.num_samps = SwapINT32(record->n);
	if (record->n < 32767)
		hdr.sampleLength = SwapINT16((INT16)record->n);
	else
		hdr.sampleLength = SwapINT16(1);

	hdr.samp_rate = SwapINT32((INT32)((1.0 / record->rate) * 1000000.0));
	if (((1.0 / record->rate) * 1000000.0) <= 32767.0)
		hdr.deltaSample = SwapINT16((INT16)((1.0 / record->rate) * 1000000.0));
	else
		hdr.deltaSample = SwapINT16(1);

	DecodeUSTime(record->ist, &year, &doy, &hour, &minute, &second);

	hdr.year = SwapINT16((INT16)year);
	hdr.day = SwapINT16((INT16)doy);
	hdr.hour = SwapINT16((INT16)hour);
	hdr.minute = SwapINT16((INT16)minute);
	hdr.second = SwapINT16((INT16)second);
	hdr.m_secs = SwapINT16((INT16)((second - (REAL32)((INT16)second)) * 1000.0));
	hdr.timeBasisCode = SwapINT16(2);

	if (record->name.station[0] != '\0')
		strncpy(hdr.station_name, record->name.station, 4);
	if (record->name.component[0] != '\0')
		strncpy(hdr.channel_name, record->name.component, 2);

	hdr.data_form = SwapINT16(1);

	/* Write to file */
	if (fwrite(&hdr, sizeof(hdr), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteSEGYData(FILE *file, RECORD * record)
{
	UINT32 i;
	INT32 *sample, isample;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	sample = record->samples;
	for (i = 0; i < record->n; i++) {
		isample = SwapINT32(sample[i]);
		if (fwrite(&isample, 4, 1, file) != 1) {
			LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
			return FALSE;
		}
	}

	return TRUE;
}
