/* $Id: stats.c,v 1.7 2002/10/02 03:18:09 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	Computes stats...

-----------------------------------------------------------------------*/

#include "ascii.h"
#include "stats.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/

/* Module macros ------------------------------------------------------*/
#define SQR(x) ((x) * (x))

/* Module prototypes --------------------------------------------------*/
static BOOL OutputStats(RECORD * record);

/*---------------------------------------------------------------------*/
BOOL ComputeStats(RECORD * head, CHAR *net_name)
{
	RECORD *record;

	ASSERT(head != NULL);

	/* Compute stats per channel */
	for (record = head; record != NULL; record = record->next) {
		if (net_name != NULL) {
			if (strcmp(record->name.network, net_name) != 0)
				continue;
		}
		if (!OutputStats(record))
			return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL OutputStats(RECORD * record)
{
	CHAR string[32];
	UINT32 i;
	INT32 min, max;
	REAL64 sample, volt, sum, mean, sigma;

	ASSERT(record != NULL);

	if (record->n < 2)
		return TRUE;

	if (IsASCIIVolts())
		volt = 1.0 / (REAL64)record->counts_per_volt;
	else
		volt = 1.0;

	sum = 0.0;
	min = 2147483647;
	max = -2147483647;

	for (i = 0; i < record->n; i++) {
		sample = (REAL64)record->samples[i] * volt;

		sum += sample;

		if (min > record->samples[i])
			min = record->samples[i];
		if (max < record->samples[i])
			max = record->samples[i];
	}

	mean = sum / (REAL64)record->n;

	sum = 0.0;
	for (i = 0; i < record->n; i++) {
		sample = (REAL64)record->samples[i] * volt;
		sum += SQR(sample - mean);
	}

	sigma = sqrt(sum / ((REAL64)record->n - 1));

	LogMessage(LOG_NOTICE, "Channel name: %s:%s:%s (%u:%u)\n", record->name.network, 
			record->name.station, record->name.component, record->unit, record->channel);
	LogMessage(LOG_NOTICE, " Time: %s (%s), rate: %.3lf sps (%s)\n",
		FormatUSTime(string, 32, UST_DATE_TIME_MSEC, record->ist),
		GRFDescribeQuality(record->timing.time_quality), record->rate, GRFDescribeQuality(record->timing.rate_quality));
	LogMessage(LOG_NOTICE, " Number of samples: %u, %.3lf seconds\n", record->n, (REAL64)record->n / record->rate);

	if (IsASCIIVolts()) {
		LogMessage(LOG_NOTICE, " Min: %lg, Max: %lg, Range: %lg, Mean: %lg, Sigma: %lg\n", 
			(REAL64)min * volt, (REAL64)max * volt, (REAL64)(max - min + 1) * volt, mean, sigma);
	}
	else {
		LogMessage(LOG_NOTICE, " Min: %d, Max %d, Range: %d, Mean: %.2lf, Sigma: %.2lf\n", 
				min, max, max - min + 1, mean, sigma);
	}

	return TRUE;
}
