/* $Id: steim.h,v 1.3 2001/09/25 20:13:21 cvs Exp $ */
/*-----------------------------------------------------------------------
    Steim 1 compression...
-----------------------------------------------------------------------*/
#if !defined _STEIM_H_INCLUDED_
#define _STEIM_H_INCLUDED_

#include "main.h"

/* SEED data record structures ----------------------------------------*/

/* SEED time structure*/
typedef struct _BTIME {
    UINT16 year;
    UINT16 day;
    UINT8 hour;
    UINT8 minute;
    UINT8 seconds;
    UINT8 align1;                           /* Align on 4 byte boundary */
    UINT16 fracs;                           /* Number of .0001 seconds */
} BTIME;

/* SEED fixed data header (occupies the first frame) */
typedef struct _DATA_HEADER {
    UINT8 seq[6];                           /* Sequence number of record */
    UINT8 type;                             /* Always a 'D' */
    UINT8 align1;                           /* Fill, always a space */
    UINT8 stn[5];                           /* Station ID (space filled) */
    UINT8 loc[2];                           /* Array/Extended Station (space filled) */
    UINT8 chn[3];                           /* Channel ID (space filled) */
    UINT8 net[2];                           /* Extended Network Type */
    BTIME start_time;                       /* Start time of record */
    UINT16 nsamps;                          /* Number of samples in record */
    INT16 rate_fac;                         /* Sample rate factor */
    INT16 rate_mul;                         /* Sample rate Multiplier */
    UINT8 act_flags;                        /* Activity Information */
    UINT8 io_flags;                         /* I/O Information */
    UINT8 qual_flags;                       /* Data Quality Information */
    UINT8 total_blks;                       /* Number blockettes to follow */
    INT32 time_correct;                     /* Number of .0001 sec correction */
    UINT16 data_start;                      /* Byte where data starts */
    UINT16 first_blk;                       /* Byte of first blockette */
} DATA_HEADER;

/* DFA state constants ------------------------------------------------ */
#define _START_STATE   0                    /* Ground state */
#define _D1            1                    /* 1 BYTE difference */
#define _D2            2                    /* 1 WORD difference */
#define _D1_D1         3                    /* 2 BYTE differences */
#define _D1_D1_D1      4                    /* 3 BYTE differences */
#define _D4_f          5                    /* 1 LONG difference  (final state) */
#define _D2_D2_f       6                    /* 2 WORD differences (final state) */
#define _D1_D1_D1_D1_f 7                    /* 4 BYTE differences (final state) */

/* Chunk types */
#define CHUNK_NULL     0L                   /* No data */
#define CHUNK_BYTES    1L                   /* 4 BYTE values */
#define CHUNK_WORDS    2L                   /* 2 WORD values */
#define CHUNK_LONG     3L                   /* 1 LONG value */

/* Steim structures and unions ---------------------------------------- */

/* Chunk union, accessible as bytes, INT16s or INT32 */
typedef union _CHUNK {
    INT8 b[4];                              /* 4 INT8 values */
    INT16 w[2];                             /* 2 INT16 values */
    INT32 l;                                /* 1 INT32 value */
} CHUNK;

/* Frame structure */
typedef struct _FRAME {
    UINT32 flags;                           /* Frame control header */
    CHUNK chunk[15];                        /* Array of chunks composing 1 frame */
} FRAME;

/* Steim compressed data record */
typedef struct _STEIM {
    FRAME frame[64];                        /* Array of frames composing the 1 record */
} STEIM;

/* DFA transition table entry */
typedef struct _TRANSITION {
    INT32 new_state;                         /* New state after transition */
    INT32 unget;                             /* Number of difs to unget */
    INT32 d_ndx;                             /* Index into buffer to place dif */
} TRANSITION;

/* Data state */
typedef struct _DATA_STATE {
    INT32 x0;                               /* Forward integrating constant (x-0) */
    INT32 xn;                               /* Reverse integrating constant (x-n) */
    UINT32 flags;                           /* Current frame flags storage */
    INT16 f_ndx;                            /* Current frame number within record */
    INT16 c_ndx;                            /* Current chunk number within frame */
} DATA_STATE;

/* Prototypes --------------------------------------------------------- */
INT16 EncodeSteim1(INT32 * samples, INT16 n_rawsamp, VOID * ptr);
INT16 DecodeSteim1(VOID * ptr, INT32 * samples);

#endif
