/* $Id: suds.c,v 1.7 2003/03/22 21:16:35 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	SUDS output...

-----------------------------------------------------------------------*/

#include "suds.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/
static CHAR name_format[MAX_PATH_LEN + 1];

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static VOID SUDSName(SUDS_STATIDENT * st, RECORD * record);
static BOOL WriteStationcomp(FILE *file, RECORD * record);
static BOOL WriteDescriptrace(FILE *file, RECORD * record);
static BOOL WriteData(FILE *file, RECORD * record);

/*---------------------------------------------------------------------*/
VOID SUDSNameFormat(CHAR *format)
{
	strcpy(name_format, format);
	return;
}

/*---------------------------------------------------------------------*/
BOOL SUDSConvert(RECORD * head, CHAR *net_name)
{
	CHAR filename[MAX_PATH_LEN + 1];
	FILE *file;
	RECORD *record;

	ASSERT(head != NULL);

	OutputFilename(head, net_name, name_format, filename);
	LogMessage(LOG_INFO, "%s\n", filename);
	if ((file = fopen(filename, "w+b")) == NULL) {
		LogMessage(LOG_ERR, "ERROR: OutputSUDS: fopen(%s) failed: %s\n", filename, strerror(errno));
		return FALSE;
	}

	/* Output an SUDS structures */
	for (record = head; record != NULL; record = record->next) {
		if (net_name != NULL) {
			if (strcmp(record->name.network, net_name) != 0)
				continue;
		}
		if (!WriteStationcomp(file, record))
			break;
		if (!WriteDescriptrace(file, record))
			break;
		if (!WriteData(file, record))
			break;
	}

	fclose(file);

	AppendFilenameToList(filename);

	return TRUE;
}

/*---------------------------------------------------------------------*/
static VOID SUDSName(SUDS_STATIDENT * st, RECORD * record)
{
	CHAR *ptr;
	size_t i;

	ASSERT(st != NULL);
	ASSERT(record != NULL);

	memset(st, 0, sizeof(SUDS_STATIDENT));

	/* Copy first 3 characters of network name... */
	strncpy(st->network, record->name.network, 3);

	/* Copy last three characters of station name with channel number appended */
	i = strlen(record->name.station);
	if (i > 3)
		ptr = &record->name.station[i - 3];
	else
		ptr = record->name.station;
	sprintf(st->st_name, "%s%u", ptr, record->channel);

	/* Copy last character of component */
	i = strlen(record->name.component);
	if (i > 0)
		st->component = tolower(record->name.component[i - 1]);
	else
		st->component = '_';

	return;
}

/*---------------------------------------------------------------------*/
static BOOL WriteStationcomp(FILE *file, RECORD * record)
{
	SUDS_STRUCTTAG tag;
	SUDS_STATIONCOMP sc;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	/* Write the tag... */
	memset(&tag, 0, sizeof(SUDS_STRUCTTAG));
	tag.sync = ST_MAGIC;
	tag.machine = MACHINE;
	tag.id_struct = STATIONCOMP;
	tag.len_struct = sizeof(SUDS_STATIONCOMP);
	tag.len_data = 0;
	if (fwrite(&tag, sizeof(SUDS_STRUCTTAG), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	/* Write the struct... */
	memset(&sc, 0, sizeof(SUDS_STATIONCOMP));
	SUDSName(&sc.sc_name, record);
	sc.azim = -32767;
	sc.incid = -32767;
	sc.st_lat = -32767.0;
	sc.st_long = -32767.0;
	sc.elev = (REAL32)-32767.0;
	sc.enclosure = '_';
	sc.annotation = 0;
	sc.recorder = '_';
	sc.rockclass = '_';
	sc.rocktype = 0;
	sc.sitecondition = '_';
	sc.sensor_type = '_';
	sc.data_type = '2';
	sc.data_units = 'd';
	sc.polarity = '_';
	sc.st_status = 0;
	sc.max_gain = (REAL32)-32767.0;
	sc.clip_value = (REAL32)-32767.0;
	sc.con_mvolts = (REAL32)-32767.0;
	sc.atod_gain = 1;
	sc.channel = record->channel;
	sc.effective = (INT32)(record->ist / UST_SECOND);
	sc.clock_correct = (REAL32)-32767.0;
	sc.station_delay = (REAL32)-32767.0;

	strcpy(sc.li_name.network, record->name.network);
	strcpy(sc.li_name.station, record->name.station);
	strcpy(sc.li_name.component, record->name.component);

	if (fwrite(&sc, sizeof(SUDS_STATIONCOMP), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteDescriptrace(FILE *file, RECORD * record)
{
	SUDS_STRUCTTAG tag;
	SUDS_DESCRIPTRACE dt;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	/* Write the tag... */
	memset(&tag, 0, sizeof(SUDS_STRUCTTAG));
	tag.sync = ST_MAGIC;
	tag.machine = MACHINE;
	tag.id_struct = DESCRIPTRACE;
	tag.len_struct = sizeof(SUDS_DESCRIPTRACE);
	tag.len_data = record->n * 4;
	if (fwrite(&tag, sizeof(SUDS_STRUCTTAG), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	/* Write the struct... */
	memset(&dt, 0, sizeof(SUDS_DESCRIPTRACE));
	SUDSName(&dt.dt_name, record);
	dt.begintime = (REAL64)record->timing.time / (REAL64)UST_SECOND;
	dt.localtime = 0;
	dt.datatype = '2';
	dt.descriptor = '_';
	dt.digi_by = 0;
	dt.processed = 0;
	dt.length = record->n;
	dt.rate = (REAL32)record->timing.rate;
	dt.mindata = -8388607;
	dt.maxdata = 8388607;
	dt.avenoise = 0;
	dt.numclip = -32767;
	dt.time_correct = (REAL64)record->timing.time_correction / (REAL64)UST_SECOND;
	dt.rate_correct = (REAL32)record->timing.rate_correction;

	strcpy(dt.li_name.network, record->name.network);
	strcpy(dt.li_name.station, record->name.station);
	strcpy(dt.li_name.component, record->name.component);

	if (fwrite(&dt, sizeof(SUDS_DESCRIPTRACE), 1, file) != 1) {
		LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
		return FALSE;
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
static BOOL WriteData(FILE *file, RECORD * record)
{
	UINT32 i;
	INT32 *sample, isample;

	ASSERT(file != NULL);
	ASSERT(record != NULL);

	sample = record->samples;
	for (i = 0; i < record->n; i++) {
		isample = SwapINT32(sample[i]);
		if (fwrite(&isample, 4, 1, file) != 1) {
			LogMessage(LOG_ERR, "ERROR: write error to output file!\n");
			return FALSE;
		}
	}

	return TRUE;
}
