/* $Id: suds.h,v 1.4 2002/10/02 03:18:09 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	SUDS output...

-----------------------------------------------------------------------*/
#if !defined _SUDS_H_INCLUDED_
#define _SUDS_H_INCLUDED_

#include "main.h"

/* Constants ----------------------------------------------------------*/
#define DFL_SUDS_NAME_FMT	"%Y%d%h%m%s%ms_%N.suds"

#define ST_MAGIC		'S'					// magic character for sync in structtag

// Define code for type of machine suds is being compiled on.
// '6' is 80x86 machine
#define MACHINE         '6'

// Define symbols for missing data.
#define NODATA          "-32767."
#define NOTIME          "-2147472000"		// Dec 14, 1901 00:00 00 GMT, near largest long
#define NOCHAR          "_"
#define NOSTRG          ""
#define NOLIST          "0"

// Integer defines for standard variable types
#define CHR             1					// char 1 byte
#define MIN             2					// char number 1 byte
#define STR             3					// string
#define BTS             4					// char 1 byte
#define SHT             5					// short 2 bytes
#define LNG             6					// long 4 bytes
#define FLT             7					// float 4 bytes
#define DBL             8					// double 8 bytes
#define STI             9					// struct stat_ident 8 bytes
#define STT             10					// st_time 4 bytes
#define MST             11					// ms_time 8 bytes
#define LLT             12					// lon_lat 8 bytes
#define CAL             13					// struct calib 500 bytes
#define CPX             14					// struct complex 16 bytes
#define BTW             15					// unsigned short 2 bytes

// Structure types or identifiers
#define NO_STRUCT       0
#define STAT_IDENT      1
#define STRUCTTAG       2
#define TERMINATOR      3
#define EQUIPMENT       4
#define STATIONCOMP     5
#define MUXDATA         6
#define DESCRIPTRACE    7
#define LOCTRACE        8
#define CALIBRATION     9
#define FEATURE         10
#define RESIDUAL        11
#define EVENT           12
#define EV_DESCRIPT     13
#define ORIGIN          14
#define SERROR          15
#define FOCALMECH       16
#define MOMENT          17
#define VELMODEL        18
#define LAYERS          19
#define COMMENT         20
#define PROFILE         21
#define SHOTGATHER      22
#define CALIB           23
#define COMPLEX         24
#define TRIGGERS        25
#define TRIGSETTING     26
#define EVENTSETTING    27
#define DETECTOR        28
#define ATODINFO        29
#define TIMECORRECTION  30
#define INSTRUMENT      31
#define CHANSET         32

/* Types --------------------------------------------------------------*/

#if defined _WIN32
#	pragma pack(4)
#endif

typedef struct {
	UINT8 sync;								// The letter S. If not present, error exists.
											// Use to unscramble damaged files or tapes.
	UINT8 machine;							// code for machine writing binary file for use
											// in identifying byte order and encoding.
	INT16 id_struct;						// structure identifier: numbers defined above
	INT32 len_struct;						// structure length in bytes for fast reading
											// and to identify new versions of the structure
	INT32 len_data;							// length of data following structure in bytes
} SUDS_STRUCTTAG;

typedef struct {							// station component identifier
	CHAR network[4];						// network name
	CHAR st_name[5];						// name of station where equipment is located
	CHAR component;							// component v,n,e
	INT16 inst_type;						// instrument type
} SUDS_STATIDENT;

#define SUDS_MAX_NET_NAME	7
#define SUDS_MAX_STN_NAME	15
#define SUDS_MAX_COMP_NAME	7
typedef struct {
	CHAR network[8];	
	CHAR station[16];
	CHAR component[8];
} SUDS_LONGIDENT;

typedef struct {
	SUDS_STATIDENT dt_name;					// station component identification
	REAL64 begintime;						// time of first data sample
	INT16 localtime;						// minutes to add to GMT to get local time
	CHAR datatype;							// s = 12 bit unsigned stored as short int, 0 to 4096,
											// q = 12 bit signed stored as short int, -2048 to 2048,
											// u = 16 bit unsigned stored as short int, 0 to 65536
											// i = 16 bit signed stored as short int, -32767 to 32767,
											// 2 = 24 bit signed integer stored as long,
											// l = 32 bit signed integer stored as long,
											// r = 12 bit data, 4 lsb time stored as short int,
											// f = float (32 bit IEEE real),
											// d = double (64 bit IEEE real),
											// c = complex,
											// v = vector,
											// t = tensor
	CHAR descriptor;						// g=good, t=telemetry noise, c=calibration, etc
	INT16 digi_by;							// agency code who digitized record; 0=original
	INT16 processed;						// processing done on this waveform
	INT32 length;							// number of samples in trace
	REAL32 rate;							// samples per second
	REAL32 mindata;							// minimum value of data (type s,l,f only)
	REAL32 maxdata;							// maximum value of data (type s,l,f only)
	REAL32 avenoise;						// average value of first 200 samples (type s,l,f only)
	INT32 numclip;							// number of clipped datapoints
	REAL64 time_correct;					// time correction to be added to begintime
	REAL32 rate_correct;					// rate correction to be added to rate
	SUDS_LONGIDENT li_name;
} SUDS_DESCRIPTRACE;

typedef struct {
	SUDS_STATIDENT sc_name;					// station component identification
	INT16 azim;								// component azimuth clockwise from north
	INT16 incid;							// component angle of incidence from vertical
											//      0 is vertical, 90 is horizontal
	REAL64 st_lat;							// latitude, north is plus
	REAL64 st_long;							// longitude, east is plus
	REAL32 elev;							// elevation in meters
	CHAR enclosure;							// d=dam, n=nuclear power plant, v=underground
											//      vault, b=buried, s=on surface, etc.
	CHAR annotation;						// annotated comment code
	CHAR recorder;							// type device data recorded on
	CHAR rockclass;							// i=igneous, m=metamorphic, s=sedimentary
	INT16 rocktype;							// code for type of rock
	CHAR sitecondition;						// p=permafrost, etc.
	CHAR sensor_type;						// sensor type: d=displacement, v=velocity,
											// a=acceleration, t=time code
	CHAR data_type;							// see SUDS_DESCRIPTRACE.datatype
	CHAR data_units;						// data units: d=digital counts, v=millivolts,
											// n=nanometers (/sec or /sec/sec)
	CHAR polarity;							// n=normal, r=reversed
	CHAR st_status;							// d=dead, g=good
	REAL32 max_gain;						// maximum gain of the amplifier
	REAL32 clip_value;						// +-value of data where clipping begins
	REAL32 con_mvolts;						// conversion factor to millivolts: mv per counts
											//      0 means not defined or not appropriate
											// max_ground_motion=digital_sample*con_mvolts*
											// max_gain
	INT16 channel;							// a2d channel number
	INT16 atod_gain;						// gain of analog to digital converter
	INT32 effective;						// date/time these values became effective
	REAL32 clock_correct;					// clock correction in seconds.
	REAL32 station_delay;					// seismological station delay.
	SUDS_LONGIDENT li_name;
} SUDS_STATIONCOMP;

#if defined _WIN32
#	pragma pack()
#endif

/* Macros -------------------------------------------------------------*/

/* Prototypes ---------------------------------------------------------*/
BOOL SUDSConvert(RECORD * head, CHAR *net_name);
VOID SUDSNameFormat(CHAR *format);

#endif
