/* $Id: swap.c,v 1.5 2001/09/25 22:47:58 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	Byte order swapping routines...

-----------------------------------------------------------------------*/

#include "swap.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/
static BOOL swap = FALSE;
static UINT8 byte_order = ENDIAN_NATIVE;

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/

/*---------------------------------------------------------------------*/
VOID SetByteOrder(UINT8 order)
{

#if defined LITTLE_ENDIAN_HOST
	if (order == ENDIAN_BIG)
		swap = TRUE;
	else
		swap = FALSE;
#else
	if (order == ENDIAN_LITTLE)
		swap = TRUE;
	else
		swap = FALSE;
#endif

    byte_order = order;

	return;
}

/*---------------------------------------------------------------------*/
BOOL SwappingBytes(VOID)
{
    return swap;
}

/*---------------------------------------------------------------------*/
UINT8 GetByteOrder(VOID)
{

    if(byte_order == ENDIAN_NATIVE) {
#if defined LITTLE_ENDIAN_HOST
        byte_order = ENDIAN_LITTLE;
#else
        byte_order = ENDIAN_BIG;
#endif
    }

    return byte_order;
}

/*---------------------------------------------------------------------*/
INT16 SwapINT16(INT16 native)
{
	UINT8 temp;

	union {
		UINT8 bytes[2];
		INT16 value;
	} parts;

	if (!swap)
		return native;

	parts.value = native;

	temp = parts.bytes[0];
	parts.bytes[0] = parts.bytes[1];
	parts.bytes[1] = temp;

	return (parts.value);
}

/*---------------------------------------------------------------------*/
INT32 SwapINT32(INT32 native)
{
	UINT8 temp;

	union {
		UINT8 bytes[4];
		INT32 value;
	} parts;

	if (!swap)
		return native;

	parts.value = native;

	temp = parts.bytes[0];
	parts.bytes[0] = parts.bytes[3];
	parts.bytes[3] = temp;
	temp = parts.bytes[1];
	parts.bytes[1] = parts.bytes[2];
	parts.bytes[2] = temp;

	return (parts.value);
}

/*---------------------------------------------------------------------*/
INT64 SwapINT64(INT64 native)
{
	UINT8 temp;

	union {
		UINT8 bytes[8];
		INT64 value;
	} parts;

	if (!swap)
		return native;

	parts.value = native;

	temp = parts.bytes[0];
	parts.bytes[0] = parts.bytes[7];
	parts.bytes[7] = temp;
	temp = parts.bytes[1];
	parts.bytes[1] = parts.bytes[6];
	parts.bytes[6] = temp;
	temp = parts.bytes[2];
	parts.bytes[2] = parts.bytes[5];
	parts.bytes[5] = temp;
	temp = parts.bytes[3];
	parts.bytes[3] = parts.bytes[4];
	parts.bytes[4] = temp;

	return (parts.value);
}

/*---------------------------------------------------------------------*/
REAL32 SwapREAL32(REAL32 native)
{
	UINT8 temp;

	union {
		UINT8 bytes[4];
		REAL32 value;
	} parts;

	if (!swap)
		return native;

	parts.value = native;

	temp = parts.bytes[0];
	parts.bytes[0] = parts.bytes[3];
	parts.bytes[3] = temp;
	temp = parts.bytes[1];
	parts.bytes[1] = parts.bytes[2];
	parts.bytes[2] = temp;

	return (parts.value);
}

/*---------------------------------------------------------------------*/
REAL64 SwapREAL64(REAL64 native)
{
	UINT8 temp;

	union {
		UINT8 bytes[8];
		REAL64 value;
	} parts;

	if (!swap)
		return native;

	parts.value = native;

	temp = parts.bytes[0];
	parts.bytes[0] = parts.bytes[7];
	parts.bytes[7] = temp;
	temp = parts.bytes[1];
	parts.bytes[1] = parts.bytes[6];
	parts.bytes[6] = temp;
	temp = parts.bytes[2];
	parts.bytes[2] = parts.bytes[5];
	parts.bytes[5] = temp;
	temp = parts.bytes[3];
	parts.bytes[3] = parts.bytes[4];
	parts.bytes[4] = temp;

	return (parts.value);
}
