/* $Id: fwtdma.h,v 1.5 2001/07/12 15:37:56 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2001 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

    FreeWave TDMA...

-----------------------------------------------------------------------*/
#if !defined _FWTDMA_H_INCLUDED_
#define _FWTDMA_H_INCLUDED_

#include <platform.h>
#include <async.h>
#include "queues.h"
#include "sock.h"
#include <grf.h>

/* Constants ----------------------------------------------------------*/
#define MAX_SLAVES				32			/* Max slaves per master */

#define TDMA_BROADCAST			0x00FFFFFF	/* Broadcast to all slaves */
#define TDMA_COMMAND			0xFF000000	/* Packet is a TDMA command */
#define TDMA_VOID_ADDR			0xFFFFFFFF	/* Undefined or void address */

#define TDMA_DELIMITER			0xFF
#define TDMA_ESCAPE				0xFE

#define TDMA_MAX_SIZE			256
#define TDMA_MAX_PAYLOAD		240

#define TX_Q_DEPTH				8

#define RTT_AVG_CONSTANT		50
#define RATE_AVG_CONSTANT		10

/* Types --------------------------------------------------------------*/
typedef struct TAG_UNIT {
	USTIMER inbound;
	REAL64 mean_inbound;
	UINT32 address;
	BOOL escape;
	UINT16 head, tail;
	UINT8 queue[GRF_MAX_PACKET_LEN];
	BOOL started;
	UINT16 sequence;
} UNIT;

typedef struct TAG_SLAVE_ARGS {
	ASYNC_PARAMS async;
	size_t tx_chunk;
	USTIME tx_period;
	USTIME timeout;
	UINT32 queue_depth;
} SLAVE_ARGS;

typedef struct TAG_MASTER_ARGS {
	ASYNC_PARAMS async;
	size_t packet_length;
	USTIME timeout;
    size_t n_slaves;
    UINT32 slaves[MAX_SLAVES];
} MASTER_ARGS;

/* Macros -------------------------------------------------------------*/

/* Prototypes ---------------------------------------------------------*/

/* slave.c */
VOID TDMASlaveSetDefaults(SLAVE_ARGS * args);
VOID TDMASlaveSetParams(H_ASYNC handle, SLAVE_ARGS * args);
INT32 TDMASlaveWrite(H_ASYNC handle, GRF_PACKET * packet, BOOL block);
BOOL TDMASlaveRetransmit(H_ASYNC handle);
BOOL TDMASlaveRead(H_ASYNC handle, GRF_PACKET * packet);

BOOL EncodeGRFPacket(GRF_PACKET * packet, UINT8 *data, size_t *n);
BOOL DetectGRFPacket(UNIT * unit, GRF_PACKET * packet);

/* master.c */
VOID TDMAMasterSetParams(H_ASYNC handle, MASTER_ARGS * args);
BOOL TDMAMasterWrite(H_ASYNC handle, GRF_PACKET * packet, UINT32 address);
BOOL TDMAMasterRead(H_ASYNC handle, GRF_PACKET * packet, UINT32 *address);

/* fwtdma.c */
BOOL TDMATransmit(H_ASYNC handle, UINT32 address, VOID *data, size_t length);
BOOL TDMAReceive(H_ASYNC handle, UINT32 *address, VOID *data, size_t *length);

/* slave_thread.c */
BOOL SlaveThreadStart(SLAVE_ARGS * args, H_QUEUE empties_queue);
BOOL SlaveThreadStop(BOOL join);
BOOL SlaveThreadIsRunning(VOID);

#endif
