/* $Id: win32svc.c,v 1.4 2002/01/03 20:19:40 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2001 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

   Windows NT/2k service support...

-----------------------------------------------------------------------*/

#include <windows.h>
#include "main.h"
#include "win32svc.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/
static char serviceName[] = SERVICE_NAME;
static char displayName[] = DISPLAY_NAME;
static BOOL isService = FALSE;
static SERVICE_STATUS_HANDLE serviceStatusHandle = 0;
static DWORD currentState = SERVICE_STOPPED;

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/
static VOID ServiceCtrlHandler(DWORD controlCode);
static VOID ServiceExit(DWORD error);
static BOOL NotifySCM(DWORD dwWin32ExitCode, DWORD dwServiceSpecificExitCode, 
        DWORD dwCheckPoint, DWORD dwWaitHint);

/*---------------------------------------------------------------------*/
BOOL ServiceInstall(VOID) 
{
    VOID *errorString;
    CHAR pathName[_MAX_PATH];
    SC_HANDLE scm, service;

    /* Open the SCM database */
    scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

    /* Get the full pathname of our service's executable */
    GetModuleFileName(NULL, pathName, sizeof(pathName));
    strcat(pathName, " -service");

    /* Add this service to the SCM database */
    service = CreateService(
            scm, 
            serviceName, 
            displayName, 
            SERVICE_ALL_ACCESS,
            SERVICE_WIN32_OWN_PROCESS,
            SERVICE_AUTO_START,
            SERVICE_ERROR_NORMAL,
            pathName,
            NULL,
            NULL,
            NULL,
            NULL,
            NULL
    );
    if (service == NULL) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: CreateService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(scm);
        return FALSE;
    }

    /* Close the newly created service and the SCM */
    CloseServiceHandle(service);
    CloseServiceHandle(scm);

    printf("Successfully installed %s version %s as a service.\n", serviceName, VERSION);

    return TRUE;
}

/*---------------------------------------------------------------------*/
BOOL ServiceRemove(VOID) 
{
    VOID *errorString;
    SC_HANDLE scm, service;

    /* Open the SCM database */
    scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

    /* Open the service for delete access */
    if ((service = OpenService(scm, serviceName, DELETE)) == NULL) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: OpenService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(scm);
        return FALSE;
    }

    /* Mark the service for deletion.
       NOTE: The service is not deleted until all handles
       to it are closed and the service stops running. */
    if (!DeleteService(service)) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: DeleteService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(service);
        CloseServiceHandle(scm);
        return FALSE;
    }

    /* Close the service and the SCM */
    CloseServiceHandle(service);
    CloseServiceHandle(scm);

    fprintf(stderr, "Successfully removed service %s version %s.\n", serviceName, VERSION);

    return TRUE;
}

/*---------------------------------------------------------------------*/
BOOL ServiceStart(VOID) 
{ 
    VOID *errorString;
    BOOL result;
    SERVICE_STATUS status; 
    DWORD oldCheckPoint, startTickCount, waitTime;
    SC_HANDLE scm, service;

    /* Open the SCM database */
    scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

    /* Open the service for all access */
    if ((service = OpenService(scm, serviceName, SERVICE_ALL_ACCESS)) == NULL) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: OpenService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(scm);
        return FALSE;
    }

    if (!StartService(service, 0, NULL)) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: StartService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(service); 
        CloseServiceHandle(scm);
        return FALSE;
    }
    else 
		printf("%s service start is pending...\n", serviceName);
 
    /* Check the status until the service is no longer start pending. */
    if (!QueryServiceStatus(service, &status)) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: StartService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(service); 
        CloseServiceHandle(scm);
        return FALSE;
    }
 
    startTickCount = GetTickCount();
    while (status.dwCurrentState == SERVICE_START_PENDING) { 
        oldCheckPoint = status.dwCheckPoint; 
 
        if ((GetTickCount() - startTickCount) > status.dwWaitHint)
            break;
 
        waitTime = status.dwWaitHint / 10;

        if( waitTime < 1000 )
            waitTime = 1000;
        else if ( waitTime > 10000 )
            waitTime = 10000;

        Sleep( waitTime );

        if (!QueryServiceStatus(service, &status)) {
		    FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                    | FORMAT_MESSAGE_FROM_SYSTEM 
                    | FORMAT_MESSAGE_IGNORE_INSERTS,
			        NULL, 
                    GetLastError(), 
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                    (LPTSTR)&errorString, 
                    0, 
                    NULL
            );
		    fprintf(stderr, "ERROR: StartService(%s): %s", serviceName, errorString);
		    LocalFree(errorString);
            break;
        }

        if (oldCheckPoint >= status.dwCheckPoint) 
            break; 
    } 
 
    if (status.dwCurrentState == SERVICE_RUNNING) {
		printf("%s version %s service is running.\n", serviceName, VERSION);
        result = TRUE;
    } else  { 
		fprintf(stderr, "%s version %s service failed to start!\n", serviceName, VERSION);
        fprintf(stderr, "  Current State: %d\n", status.dwCurrentState); 
        fprintf(stderr, "  Exit Code: %d\n", status.dwWin32ExitCode); 
        fprintf(stderr, "  Service Specific Exit Code: %d\n", 
            status.dwServiceSpecificExitCode); 
        fprintf(stderr, "  Check Point: %d\n", status.dwCheckPoint); 
        fprintf(stderr, "  Wait Hint: %d\n", status.dwWaitHint); 
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "  Last error: %s", errorString);
		LocalFree(errorString);
        result = FALSE;
    } 
 
    CloseServiceHandle(service); 
    CloseServiceHandle(scm);

    return result;
} 

/*---------------------------------------------------------------------*/
BOOL ServiceStop(VOID) 
{ 
    VOID *errorString;
    SERVICE_STATUS status; 
    SC_HANDLE scm, service;

    /* Open the SCM database */
    scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

    /* Open the service for all access */
    if ((service = OpenService(scm, serviceName, SERVICE_ALL_ACCESS)) == NULL) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: OpenService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(scm);
        return FALSE;
    }

    if (!ControlService(service, SERVICE_CONTROL_STOP, &status)) {
		fprintf(stderr, "%s service failed to stop!\n", serviceName);
        fprintf(stderr, "  Current State: %d\n", status.dwCurrentState); 
        fprintf(stderr, "  Exit Code: %d\n", status.dwWin32ExitCode); 
        fprintf(stderr, "  Service Specific Exit Code: %d\n", 
            status.dwServiceSpecificExitCode); 
        fprintf(stderr, "  Check Point: %d\n", status.dwCheckPoint); 
        fprintf(stderr, "  Wait Hint: %d\n", status.dwWaitHint); 
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "  Last error: %s", errorString);
		LocalFree(errorString);
        CloseServiceHandle(service); 
        CloseServiceHandle(scm);
        return FALSE;
    }

    CloseServiceHandle(service); 
    CloseServiceHandle(scm);

    printf("%s service stop is pending...\n", serviceName);
    return TRUE;
}

/*---------------------------------------------------------------------*/
BOOL ServiceStatus(VOID) 
{ 
    VOID *errorString;
    CHAR state[32];
    SERVICE_STATUS status; 
    SC_HANDLE scm, service;

    /* Open the SCM database */
    scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

    /* Open the service for all access */
    if ((service = OpenService(scm, serviceName, SERVICE_ALL_ACCESS)) == NULL) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: OpenService(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(scm);
        return FALSE;
    }

    /* Check the status. */
    if (!QueryServiceStatus(service, &status)) {
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER 
                | FORMAT_MESSAGE_FROM_SYSTEM 
                | FORMAT_MESSAGE_IGNORE_INSERTS,
			    NULL, 
                GetLastError(), 
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
                (LPTSTR)&errorString, 
                0, 
                NULL
        );
		fprintf(stderr, "ERROR: QueryServiceStatus(%s): %s", serviceName, errorString);
		LocalFree(errorString);
        CloseServiceHandle(service); 
        CloseServiceHandle(scm);
        return FALSE;
    }

    switch (status.dwCurrentState) {
      case SERVICE_STOPPED:
        strcpy(state, "stopped");
        break;
      case SERVICE_START_PENDING:
        strcpy(state, "starting");
        break;
      case SERVICE_STOP_PENDING:
        strcpy(state, "stopping");
        break;
      case SERVICE_RUNNING:
        strcpy(state, "running");
        break;
      case SERVICE_CONTINUE_PENDING:
        strcpy(state, "continuing");
        break;
      case SERVICE_PAUSE_PENDING:
        strcpy(state, "pausing");
        break;
      case SERVICE_PAUSED:
        strcpy(state, "paused");
        break; 
      default:
        strcpy(state, "in and unknown state");
        break;
    }
    
	printf("%s version %s service is currently %s.\n", serviceName, VERSION, state);
 
    CloseServiceHandle(service); 
    CloseServiceHandle(scm);

    return TRUE;
} 

/*---------------------------------------------------------------------*/
BOOL IsService(VOID) 
{
    return isService;
}

/*---------------------------------------------------------------------*/
VOID WINAPI ServiceMain(DWORD argc, LPTSTR * argv)
{
    MAIN_ARGS args;

    /* We'll be running as a service */
    isService = TRUE;

    /* Install the service control handler */
    serviceStatusHandle = RegisterServiceCtrlHandler(argv[0], (LPHANDLER_FUNCTION)ServiceCtrlHandler);
    if (!serviceStatusHandle) {
        ServiceDebugMessage("%s: ServiceMain: RegisterServiceCtrlHandler() failed: %d\n", 
                serviceName, GetLastError()); 
        ServiceExit(1);
        return;
    }

    currentState = SERVICE_RUNNING;
    if (!NotifySCM(NO_ERROR, 0, 0, 0)) {
        ServiceDebugMessage("%s: ServiceMain: NotifySCM(RUNNING, 0) failed!\n",
                serviceName); 
        ServiceExit(6);
        return;
    }

    /* Run the main thread with no args */
	args.argc = argc;
	args.argv = argv;
    MainThread(&args);

    currentState = SERVICE_STOPPED;
    if (!NotifySCM(NOERROR, 0, 0, 0))
        ServiceDebugMessage(" %s: ServiceExit: NotifySCM(STOPPED, 0) failed!\n", 
                serviceName); 
    return;
}

/*---------------------------------------------------------------------*/
static VOID ServiceCtrlHandler(DWORD controlCode)
{
    DWORD currentState = 0;

    /* Deal with events received from the SCM */
    switch (controlCode) {

      case SERVICE_CONTROL_STOP:
      case SERVICE_CONTROL_SHUTDOWN:
        ServiceExit(0);
        return;

      case SERVICE_CONTROL_INTERROGATE:
        if (!NotifySCM(NO_ERROR, 0, 0, 0))
            ServiceDebugMessage(" %s: Handler: NotifySCM(NO_ERROR, 0) failed!\n",
                    serviceName); 
        return;

    }
}

/*---------------------------------------------------------------------*/
static VOID ServiceExit(DWORD error)
{
    /* Bring down any and all application threads */
    RequestShutDown(TRUE);

    /* Notify SCM that we are going down */
    if (serviceStatusHandle) {
        currentState = SERVICE_STOP_PENDING;
        if (!NotifySCM(NO_ERROR, 0, 1, 10000))
            ServiceDebugMessage("%s: ServiceExit: NotifySCM(STOP_PENDING, 1) failed!\n",
                    serviceName); 
    }
}

/*---------------------------------------------------------------------*/
static BOOL NotifySCM(DWORD dwWin32ExitCode, DWORD dwServiceSpecificExitCode, 
        DWORD dwCheckPoint, DWORD dwWaitHint)
{
    SERVICE_STATUS status;

    /* General purpose SCM reports */
    status.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
    status.dwCurrentState = currentState;

    if (currentState == SERVICE_START_PENDING) {
        status.dwControlsAccepted = 0;
    } else {
        status.dwControlsAccepted = SERVICE_ACCEPT_STOP;
        status.dwControlsAccepted |= SERVICE_ACCEPT_SHUTDOWN;
    }

    if (dwServiceSpecificExitCode == 0) {
        status.dwWin32ExitCode = dwWin32ExitCode;
    } else {
        status.dwWin32ExitCode = ERROR_SERVICE_SPECIFIC_ERROR;
    }
    status.dwServiceSpecificExitCode = dwServiceSpecificExitCode;

    status.dwCheckPoint = dwCheckPoint;
    status.dwWaitHint = dwWaitHint;

    if (!SetServiceStatus(serviceStatusHandle, &status)) {
        ServiceDebugMessage(" %s: SetServiceStatus: error: %d\n", 
                serviceName, GetLastError()); 
        ServiceExit(3);
        return FALSE;
    }

    return TRUE;
}

/*---------------------------------------------------------------------*/
VOID ServiceDebugMessage(CHAR *format, ...) 
{
    CHAR string[MAX_MESSAGE_LENGTH + 1];
    va_list marker;

    va_start(marker, format);
    _vsnprintf(string, MAX_MESSAGE_LENGTH, format, marker);
    va_end(marker);
    string[MAX_MESSAGE_LENGTH] = '\0';

    OutputDebugStringA(string); 
} 
