/* $Id: threads.c,v 1.2 2001/06/29 16:04:32 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

    Thread management...

-----------------------------------------------------------------------*/

#include <platform.h>
#include "threads.h"
#include "mem.h"
#include "log.h"

/* Module constants ---------------------------------------------------*/

/* Module types -------------------------------------------------------*/

/* Module globals -----------------------------------------------------*/
static BOOL initialized = FALSE;
static MUTEX mutex;
static BOOL shut_down = FALSE;

/* Module macros ------------------------------------------------------*/

/* Module prototypes --------------------------------------------------*/

/*---------------------------------------------------------------------*/
VOID RequestShutDown(BOOL state)
{
	if (!initialized) {
		MUTEX_INIT(&mutex);
		initialized = TRUE;
	}

	MUTEX_LOCK(&mutex);
	shut_down = state;
	MUTEX_UNLOCK(&mutex);

	return;
}

/*---------------------------------------------------------------------*/
BOOL ShutDownRequested(VOID)
{
	BOOL result;

	if (!initialized) {
		MUTEX_INIT(&mutex);
		initialized = TRUE;
	}

	MUTEX_LOCK(&mutex);
	result = shut_down;
	MUTEX_UNLOCK(&mutex);

	return result;
}

/*---------------------------------------------------------------------*/
VOID ThreadListInit(THREAD_LIST * list)
{
	ASSERT(list != NULL);

	MUTEX_INIT(&list->mutex);
	list->head = NULL;

	return;
}

/*---------------------------------------------------------------------*/
BOOL ThreadRegister(THREAD_LIST * list, THREAD thread_id, CHAR *name)
{
	THREAD_NODE *node;

	ASSERT(list != NULL);
	ASSERT(name != NULL);

	/* Allocate a new node */
	if (!AllocateMemory((void **)&node, sizeof(THREAD_NODE))) {
		LogMessage(LOG_ERR, "ERROR: ThreadRegister: Unable to allocate memory!\n");
		return FALSE;
	}

	/* Fill it in */
	node->thread_id = thread_id;
	strncpy(node->name, name, MAX_THREAD_NAME);
	node->name[MAX_THREAD_NAME] = '\0';

	LogMessage(LOG_DEBUG, "ThreadRegister: %s (%d) registered by %d\n", node->name, node->thread_id, THREAD_SELF());

	/* Link into list at head */
	MUTEX_LOCK(&list->mutex);
	node->next = list->head;
	list->head = node;
	MUTEX_UNLOCK(&list->mutex);

	return TRUE;
}

/*---------------------------------------------------------------------*/
BOOL ThreadJoinAll(THREAD_LIST * list)
{
	CHAR name[MAX_THREAD_NAME + 1];
	THREAD thread_id;
	THREAD_NODE *node;

	ASSERT(list != NULL);

	while (TRUE) {
		/* Get the first thread in the list */
		MUTEX_LOCK(&list->mutex);
		if (list->head != NULL) {
			thread_id = list->head->thread_id;
			strcpy(name, list->head->name);
			/* Unlink and release */
			node = list->head;
			list->head = list->head->next;
			ReleaseMemory(node);
		}
		else {
			MUTEX_UNLOCK(&list->mutex);
			break;
		}
		MUTEX_UNLOCK(&list->mutex);

		/* Join it */
		LogMessage(LOG_DEBUG, "Thread %d joining %s (%d)\n", THREAD_SELF(), name, thread_id);
		THREAD_JOIN(thread_id);
	}

	return TRUE;
}

/*---------------------------------------------------------------------*/
VOID ThreadLogAll(THREAD_LIST * list, UINT32 log_level)
{
	INT32 count;
	THREAD_NODE *node;

	ASSERT(list != NULL);

	count = 0;
	LogMessage(log_level, "Threads registered to 0x%x:\n", THREAD_SELF());

	MUTEX_LOCK(&list->mutex);

	for (node = list->head; node != NULL; node = node->next)
		LogMessage(log_level, " %2d %s (%d)\n", count + 1, node->name, node->thread_id);

	MUTEX_UNLOCK(&list->mutex);


	return;
}
