/* $Id: trigger.h,v 1.3 2002/03/15 00:03:14 cvs Exp $ */
/*-----------------------------------------------------------------------
    Copyright (C) 2000 - Robert Banfill - All rights reserved.
-------------------------------------------------------------------------

	Main include for grftrig...

-----------------------------------------------------------------------*/
#if !defined _MAIN_H_INCLUDED_
#define _MAIN_H_INCLUDED_

#include <platform.h>
#include <ustime.h>
#include "sock.h"
#include <grf.h>

/* Constants ----------------------------------------------------------*/

/* Trigger types... */
#define TRIG_NONE			0
#define TRIG_AMP			1
#define TRIG_STALTA			2
#define TRIG_MAX			2

/* Trigger states... */
#define STA_UNINITIALIZED	0
#define STA_UNTRIGGERED		1
#define STA_TRIGGER			2
#define STA_TRIGGERED       3
#define STA_DETRIGGER       4
#define STA_MAX				4

/* Types --------------------------------------------------------------*/

typedef struct TAG_PL_NODE {
	struct TAG_PL_NODE *prev;
	struct TAG_PL_NODE *next;
	USTIME ist;
	USTIME lst;
	USTIME period;
	GRF_PACKET packet;
} PL_NODE;

typedef struct TAG_PACKET_LIST {
	PL_NODE head;
	PL_NODE tail;
} PACKET_LIST;

typedef struct TAG_TRIGGER {
	UINT8 type;								/* See TRIG_? constants above */
	UINT8 state;							/* See STA_? constants above */
	USTIME period;							/* Sampling period in microseconds */
	USTIME time;							/* Time of last packet */
	USTIME trig_time;						/* Trigger time */
	USTIME detrig_time;						/* Detrigger time */
	union {
		struct TAG_AMP {
			/* Parameters... */
			USTIME mean_sec;				/* Running mean time constant */
			INT32 trig_amp;					/* Trigger amplitude in counts */
			USTIME trig_sec;				/* Trigger count */
			INT32 detrig_amp;				/* Detrigger amplitude in counts */
			USTIME detrig_sec;				/* Detrigger count */

			/* State data... */
			REAL32 mean;
			UINT32 mean_n;
			UINT32 trig_n;
			UINT32 detrig_n;
			UINT32 count;
			UINT32 holdoff;
			UINT32 n;
			UINT32 max_n;
		} amp;
		struct TAG_STALTA {
			/* Parameters... */
			USTIME mean_sec;				/* Running mean time constant */
			USTIME sta_sec;					/* STA length */
			USTIME lta_sec;					/* LTA length */
			REAL32 trig_ratio;				/* STA/LTA trigger ratio */
			REAL32 detrig_ratio;			/* STA/LTA detrigger ratio */
			BOOL lta_hold;					/* Freeze LTA on trigger? */

			/* State data... */
			REAL32 mean;
			UINT32 mean_n;
			REAL32 sta;
			UINT32 sta_n;
			REAL32 lta;
			UINT32 lta_n;
			REAL32 ratio;
			UINT32 holdoff;
			UINT32 n;
			UINT32 max_n;
		} stalta;
	} prm;
} TRIGGER;

typedef struct TAG_VOTER {
	struct TAG_VOTER *next;
	GRF_NAME name;							/* Channel name */
	TRIGGER trigger;						/* Trigger state */
} VOTER;

typedef struct TAG_CHANNEL {
	struct TAG_CHANNEL *next;
	BOOL active;
	UINT32 unit;							/* GRF unit number */
	UINT16 channel;							/* GRF channel number */
	REAL64 rate;							/* Samples/second */
	GRF_NAME name;							/* Channel name */
	USTIME latest;
	PACKET_LIST packets;
} CHANNEL;

typedef struct TAG_NETWORK {
	struct TAG_NETWORK *next;
	/* Parameters... */
	UINT32 trig_votes;						/* Channel triggers required for network trigger */
	UINT32 detrig_votes;					/* Channel triggers remaining for network detrigger */
	USTIME window;							/* Triggering coincidence window */
	USTIME pre_event;						/* Pre event length */
	USTIME post_event;						/* Post event length */
	USTIME min_event;						/* Minimum length */
	USTIME max_event;						/* Maximum length */
	CHAR name[GRF_MAX_NET_LEN + 1];			/* Network name */

	/* State info... */
	VOTER *voters;							/* Channel triggers... */
	CHANNEL *channels;						/* Channels to record */
	USTIME time;							/* Master time */
	UINT8 state;							/* See STA_? constants above */
	USTIME trig_time;						/* Network trigger time */
	USTIME detrig_time;						/* Network detrigger time */
	BOOL recording;
	BOOL msg_trigger;					
} NETWORK;

/* Macros -------------------------------------------------------------*/

/* Prototypes ---------------------------------------------------------*/
VOID SetNetworkDefaults(NETWORK * net);
VOID SetChannelDefaults(CHANNEL * chan);
VOID SetTriggerDefaults(TRIGGER * trigger, UINT8 type);
BOOL HandlePacket(NETWORK * networks, GRF_PACKET * packet);

#endif
